@@ -172,7 +172,7 @@ gb_clipboard_get_cb (GtkClipboard *clipboard,
 		{
 			g_return_if_fail (data);
 			gtk_selection_data_set (selection_data, gdk_atom_intern (XBEL_TARGET_ID , FALSE),
-						8, data, strlen (data));
+						8, (guchar*)data, strlen (data));
 			g_free (data);
 		}
 		g_object_unref (io);
@@ -210,7 +210,7 @@ gb_clipboard_paste (GbClipboard *cb)
 	if (data)
 	{
 		GbIO *io = GB_IO (gb_xbel_new ());
-		GbBookmarkSet *ret = gb_io_load_from_string (io, data->data);
+		GbBookmarkSet *ret = gb_io_load_from_string (io, (char*)data->data);
 		gtk_selection_data_free (data);
 		g_object_unref (io);
 		return ret;
