@@ -18,20 +18,21 @@
 
 #include "galeon-embed-popup.h"
 #include "galeon-embed-event.h"
+#include "galeon-embed-shell.h"
 #include "galeon-embed-utils.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 #include "galeon-config.h"
 #include "gul-bonobo-extensions.h"
-#include "gul-download.h"
 #include "gul-string.h"
 #include "galeon-embed-helper-list.h"
 #include "galeon-marshal.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <gtk/gtkclipboard.h>
+#include <gtk/gtkmain.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
@@ -43,6 +44,10 @@ typedef enum
 	EMBED_POPUP_SELECTION
 } EmbedPopupType;
 
+#define GALEON_EMBED_POPUP_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_EMBED_POPUP, GaleonEmbedPopupPrivate))
+
+
 struct GaleonEmbedPopupPrivate
 {
 	GaleonEmbedHelperList *helper_list;
@@ -100,11 +105,6 @@ embed_popup_open_frame_cmd (BonoboUIComponent *uic,
                       	    GaleonEmbedPopup *popup, 
                             const char* verbname);
 static void 
-embed_popup_reload_frame_cmd (BonoboUIComponent *uic, 
-                      	      GaleonEmbedPopup *popup, 
-                              const char* verbname);
-
-static void 
 embed_popup_open_image_cmd (BonoboUIComponent *uic, 
                       	    GaleonEmbedPopup *popup, 
                             const char* verbname);
@@ -143,6 +143,7 @@ static gint GaleonEmbedPopupSignals[GALEON_EMBED_POPUP_LAST_SIGNAL];
  *text must be done here
  */
 #define GALEON_POPUP_BLOCK_IMAGE_SITE_PATH "/commands/EPBlockImageSite"
+#define GALEON_POPUP_ALLOW_IMAGE_SITE_PATH "/commands/EPAllowImageSite"
 
 BonoboUIVerb embed_popup_verbs [] = {
 	BONOBO_UI_VERB ("EPCopyLinkLocation", (BonoboUIVerbFn)embed_popup_copy_link_location_cmd),
@@ -157,7 +158,6 @@ BonoboUIVerb embed_popup_verbs [] = {
         BONOBO_UI_VERB ("DPSavePageAs", (BonoboUIVerbFn)embed_popup_save_page_as_cmd),
 	BONOBO_UI_VERB ("DPSaveBackgroundAs", (BonoboUIVerbFn)embed_popup_save_background_as_cmd),
 	BONOBO_UI_VERB ("DPOpenFrame", (BonoboUIVerbFn)embed_popup_open_frame_cmd),
-	BONOBO_UI_VERB ("DPReloadFrame", (BonoboUIVerbFn)embed_popup_reload_frame_cmd),
 	
 	BONOBO_UI_VERB_END
 };
@@ -210,12 +210,14 @@ galeon_embed_popup_class_init (GaleonEmbedPopupClass *klass)
 		galeon_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
 
+
+	g_type_class_add_private (klass, sizeof (GaleonEmbedPopupPrivate));
 }
 
 static void
 galeon_embed_popup_init (GaleonEmbedPopup *gep)
 {
-        gep->priv = g_new0 (GaleonEmbedPopupPrivate, 1);
+        gep->priv = GALEON_EMBED_POPUP_GET_PRIVATE (gep);
 	gep->priv->embed = NULL;
 	gep->priv->event = NULL;
 	gep->priv->ui_component = NULL;
@@ -228,7 +230,7 @@ galeon_embed_popup_finalize (GObject *object)
 	GaleonEmbedPopup *gep;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_GALEON_EMBED_POPUP (object));
+        g_return_if_fail (GALEON_IS_EMBED_POPUP (object));
 
         gep = GALEON_EMBED_POPUP (object);
 
@@ -239,7 +241,11 @@ galeon_embed_popup_finalize (GObject *object)
 		g_object_unref (G_OBJECT (gep->priv->event));
 	}	
 	
-        g_free (gep->priv);
+	if (gep->priv->helper_list)
+	{
+		g_object_unref (G_OBJECT (gep->priv->helper_list));
+	}
+	
 	
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -253,7 +259,7 @@ setup_image_openwith (GaleonEmbedPopup *p)
 	GaleonEmbedHelperList *hl;
 	
 	info = galeon_embed_popup_get_event (p);
-	galeon_embed_event_get_property (info, "image", &value);
+	value = galeon_embed_event_get_property (info, "image");
 	url = g_value_get_string (value);
 		
 	hl = p->priv->helper_list;
@@ -262,8 +268,8 @@ setup_image_openwith (GaleonEmbedPopup *p)
 		/* it must be an image, so let's try with png */
 		galeon_embed_helper_list_add_mime_type (hl, "image/png");
 	}
-	galeon_embed_helper_list_add_to_menu (hl, p->priv->ui_component,
-					      GALEON_POPUP_IMAGE_OPENWITH_PATH);
+	galeon_embed_helper_list_add_to_bonoboui_menu (hl, p->priv->ui_component,
+						       GALEON_POPUP_IMAGE_OPENWITH_PATH);
 }
 
 static void 
@@ -305,10 +311,12 @@ setup_element_menu (GaleonEmbedPopup *p)
 		     *esc_file = NULL, *file = NULL;
 		const char *host = NULL;
 		GnomeVFSURI *uri = NULL;
+		const char *show_path, *hide_path;
+		PermissionActionType perm_action;
 
 		info = galeon_embed_popup_get_event(p);
 
-		galeon_embed_event_get_property(info, "image", &value);
+		value = galeon_embed_event_get_property(info, "image");
 
 		uri = gnome_vfs_uri_new(g_value_get_string(value));
 
@@ -323,31 +331,54 @@ setup_element_menu (GaleonEmbedPopup *p)
 
 		if (file)
 		{
-			esc_file = gul_string_escape_uline_accel(file);
+			esc_file = gul_string_double_underscores (file);
 			g_free(file);
 		}
 
-		block_host_str = g_strdup_printf(_("Block Images From %s"),
-						 host);
+		perm_action = galeon_embed_shell_test_permission(embed_shell,
+								 g_value_get_string(value),
+								 IMAGES_PERMISSION);
+
+		if (perm_action == DENY_ACTION)
+		{
+			block_host_str = g_strdup_printf(_("Allow Images From %s"),
+							 host);
+			show_path = GALEON_POPUP_ALLOW_IMAGE_SITE_PATH;
+			hide_path = GALEON_POPUP_BLOCK_IMAGE_SITE_PATH;
+		}
+		else
+		{
+			block_host_str = g_strdup_printf(_("Block Images From %s"),
+							 host);
+			hide_path = GALEON_POPUP_ALLOW_IMAGE_SITE_PATH;
+			show_path = GALEON_POPUP_BLOCK_IMAGE_SITE_PATH;
+		}
+
+		gul_bonobo_set_label(p->priv->ui_component,
+				     show_path,
+				     block_host_str);
+		gul_bonobo_set_hidden(p->priv->ui_component,
+				      show_path, FALSE);
+		gul_bonobo_set_hidden(p->priv->ui_component,
+				      hide_path, TRUE);
+
 		open_image_str = g_strdup_printf(_("Open Image (%s)"),
 						 esc_file);
+
+		gul_bonobo_set_label(p->priv->ui_component,
+				     GALEON_POPUP_OPEN_IMAGE_PATH,
+				     open_image_str);
+
 		if (esc_file)
 			g_free(esc_file);
 
 		if (uri)
 			gnome_vfs_uri_unref(uri);
 
-		gul_bonobo_set_label(p->priv->ui_component,
-				     GALEON_POPUP_OPEN_IMAGE_PATH,
-				     open_image_str);
-		gul_bonobo_set_label(p->priv->ui_component,
-				     GALEON_POPUP_BLOCK_IMAGE_SITE_PATH,
-				     block_host_str);
-
 		g_free(block_host_str);
 		g_free(open_image_str);
 
-		setup_image_openwith (p);
+		setup_image_openwith(p);
 	}
 }
 
@@ -359,8 +390,7 @@ setup_doc_openwith (GaleonEmbedPopup *popup)
 
 	g_return_if_fail (popup->priv->embed != NULL);
 
-	galeon_embed_get_location (popup->priv->embed, FALSE, 
-				   FALSE, &location);
+	location = galeon_embed_get_location (popup->priv->embed, FALSE, FALSE);
 
 	if (!galeon_embed_helper_list_set_uri (hl, location))
 	{
@@ -372,8 +402,8 @@ setup_doc_openwith (GaleonEmbedPopup *popup)
 	
 	galeon_embed_helper_list_add_mime_type (hl, "text/plain");
 	
-	galeon_embed_helper_list_add_to_menu (hl, popup->priv->ui_component,
-					      GALEON_POPUP_DOC_OPENWITH_PATH);
+	galeon_embed_helper_list_add_to_bonoboui_menu (hl, popup->priv->ui_component,
+						       GALEON_POPUP_DOC_OPENWITH_PATH);
 }
 
 static void
@@ -384,8 +414,8 @@ setup_document_menu (GaleonEmbedPopup *p)
 	gboolean has_background;
 	gboolean is_sidebar = p->priv->context & EMBED_CONTEXT_SIDEBAR;
 	
-	galeon_embed_event_get_property (p->priv->event, 
-					 "framed_page", &value);
+	value = galeon_embed_event_get_property (p->priv->event,
+						 "framed_page");
 	is_framed = g_value_get_int (value);
 	gul_bonobo_set_hidden (BONOBO_UI_COMPONENT(p->priv->ui_component), 
 			       GALEON_POPUP_FRAME_ITEMS_PLACEHOLDER, !is_framed);
@@ -412,7 +442,7 @@ galeon_embed_popup_set_event (GaleonEmbedPopup *p,
 		g_object_unref (G_OBJECT (p->priv->event));
 	}
 
-	galeon_embed_event_get_context (event, &context); 
+	context = galeon_embed_event_get_context (event);
 	
 	p->priv->context = context;
 
@@ -468,7 +498,7 @@ galeon_embed_popup_show (GaleonEmbedPopup *p,
 			 GaleonEmbed *embed)
 {
 	GaleonEmbedPopupClass *klass = GALEON_EMBED_POPUP_GET_CLASS (p);
-	return klass->show (p, embed);
+	klass->show (p, embed);
 
 }
 
@@ -487,7 +517,7 @@ galeon_embed_popup_connect_verbs (GaleonEmbedPopup *p,
 GaleonEmbedEvent *
 galeon_embed_popup_get_event (GaleonEmbedPopup *p)
 {
-	g_return_val_if_fail (IS_GALEON_EMBED_POPUP (p), NULL);
+	g_return_val_if_fail (GALEON_IS_EMBED_POPUP (p), NULL);
 
 	return p->priv->event;
 }
@@ -498,8 +528,7 @@ embed_popup_copy_location_cmd (BonoboUIComponent *uic,
                                const char* verbname)
 {
 	char *location;
-	galeon_embed_get_location (popup->priv->embed, FALSE, 
-				   FALSE, &location);
+	location = galeon_embed_get_location (popup->priv->embed, FALSE, FALSE);
 	embed_popup_copy_to_clipboard (popup, location);
 	g_free (location);
 }
@@ -514,7 +543,7 @@ embed_popup_copy_email_cmd (BonoboUIComponent *uic,
 	const GValue *value;
 	
 	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, "email", &value);
+	value = galeon_embed_event_get_property (info, "email");
 	location = g_value_get_string (value);
 	embed_popup_copy_to_clipboard (popup, location);
 }
@@ -529,60 +558,11 @@ embed_popup_copy_link_location_cmd (BonoboUIComponent *uic,
 	const GValue *value;
 	
 	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, "link", &value);
+	value = galeon_embed_event_get_property (info, "link");
 	location = g_value_get_string (value);
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
-static void
-save_property_url (GaleonEmbedPopup *popup,
-		   gboolean ask_dest,
-		   gboolean show_progress,
-		   const char *property)
-{
-	GaleonEmbedEvent *info;
-	const char *location;
-	const GValue *value;
-	GtkWidget *widget;
-	GtkWidget *window;
-	GaleonEmbedPersist *persist;
-	
-	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, property, &value);
-	location = g_value_get_string (value);
-
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-	
-	persist = galeon_embed_persist_new (popup->priv->embed);
-	
-	galeon_embed_persist_set_source (persist, location);
-
-	if (show_progress)
-	{
-		galeon_embed_persist_set_flags (persist, 
-						EMBED_PERSIST_SHOW_PROGRESS);
-	}
-	
-	if (!ask_dest)
-	{
-		/* if we're not asking for download directory, make sure the
-		 * file is downloaded in the specified download directory and
-		 * not somewhere else */
-		char *dir = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-		eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR, dir);
-		g_free (dir);
-	}
-
-	galeon_embed_utils_save (window,
-				 CONF_STATE_LAST_DOWNLOAD_DIR,
-				 ask_dest,
-                                 FALSE,
-                                 persist);
-
-	g_object_unref (persist);
-}
-
 const char *
 galeon_embed_popup_get_popup_path (GaleonEmbedPopup *p)
 {
@@ -614,28 +594,10 @@ embed_popup_download_link_cmd (BonoboUIComponent *uic,
                       	       GaleonEmbedPopup *popup, 
                                const char* verbname)
 {
-        if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER) ==
-            DOWNLOADER_EXTERNAL)
-        {
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
 
-		GaleonEmbedEvent *info;
-		const char *location;
-		const GValue *value;
-
-		info = galeon_embed_popup_get_event (popup);
-		galeon_embed_event_get_property (info, "link", &value);
-		location = g_value_get_string (value);
-
-		return gul_download_external_save_url (location);
-        }
-	else
-	{
-		save_property_url (popup,
-				   eel_gconf_get_boolean 
-				   (CONF_DOWNLOADING_ASK_DIR),
-				   TRUE,
-				   "link");
-	}
+	galeon_embed_utils_download_event_property (embed, event, FALSE, NULL, "link");
 }
 
 static void 
@@ -643,7 +605,11 @@ embed_popup_save_image_as_cmd (BonoboUIComponent *uic,
                       	       GaleonEmbedPopup *popup, 
                                const char* verbname)
 {
-	save_property_url (popup, TRUE, FALSE, "image");
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
+	
+	galeon_embed_utils_save_event_property (embed, event, TRUE, FALSE, 
+						_("Save Image As..."), "image" );
 }
 
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
@@ -656,19 +622,17 @@ background_download_completed (GaleonEmbedPersist *persist,
 	const char *bg;
 	char *type;
 	
-	galeon_embed_persist_get_dest (persist, &bg);
+	bg = galeon_embed_persist_get_dest (persist);
 	eel_gconf_set_string (CONF_DESKTOP_BG_PICTURE, bg);
 
 	type = eel_gconf_get_string (CONF_DESKTOP_BG_TYPE);
-	if (type || strcmp (type, "none") == 0)
+	if (type && strcmp (type, "none") == 0)
 	{
 		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE,
 				      "wallpaper");
 	}
 
 	g_free (type);
-
-	g_object_unref (persist);
 }
 
 static void 
@@ -683,7 +647,7 @@ embed_popup_set_image_as_background_cmd (BonoboUIComponent *uic,
 	GaleonEmbedPersist *persist;
 	
 	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, "image", &value);
+	value = galeon_embed_event_get_property (info, "image");
 	location = g_value_get_string (value);
 
 	persist = galeon_embed_persist_new (popup->priv->embed);
@@ -696,12 +660,13 @@ embed_popup_set_image_as_background_cmd (BonoboUIComponent *uic,
 	galeon_embed_persist_set_source (persist, location);
 	galeon_embed_persist_set_dest (persist, dest);
 
-	galeon_embed_persist_save (persist);
-
 	g_signal_connect (persist, "completed",
 			  G_CALLBACK (background_download_completed),
 			  NULL);
 	
+	galeon_embed_persist_save (persist);
+	g_object_unref (persist);
+
 	g_free (dest);
 	g_free (base);
 }
@@ -716,61 +681,46 @@ embed_popup_copy_image_location_cmd (BonoboUIComponent *uic,
 	const GValue *value;
 	
 	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, "image", &value);
+	value = galeon_embed_event_get_property (info, "image");
 	location = g_value_get_string (value);
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
-static void
-save_url (GaleonEmbedPopup *popup,
-	  gboolean ask_dest,
-	  gboolean show_progress,
-	  const char *url)
-{
-	GtkWidget *widget;
-	GtkWidget *window;
-	GaleonEmbedPersist *persist;
-	
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-	
-	persist = galeon_embed_persist_new (popup->priv->embed);
-	galeon_embed_persist_set_source (persist, url);
-
-	if (show_progress)
-	{
-		galeon_embed_persist_set_flags (persist, 
-						EMBED_PERSIST_SHOW_PROGRESS);
-	}
-	
-	galeon_embed_utils_save (window,
-				 CONF_DOWNLOADING_DIR,
-				 ask_dest,
-                                 FALSE,
-                                 persist);
-
-	g_object_unref (persist);
-}
-
 static void 
 embed_popup_save_page_as_cmd (BonoboUIComponent *uic, 
                       	      GaleonEmbedPopup *popup, 
                               const char* verbname)
 {
-	char *location;
+	GtkWidget *window;
+	GaleonEmbedPersist *persist;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (popup->priv->embed));
 	
-	galeon_embed_get_location (popup->priv->embed,
-				   FALSE, FALSE, &location);
-	save_url (popup, TRUE, FALSE, location);
-	g_free (location);
+	persist = galeon_embed_persist_new (popup->priv->embed);
+	galeon_embed_persist_set_flags (persist, EMBED_PERSIST_MAINDOC
+					| EMBED_PERSIST_SAVE_CONTENT
+					| EMBED_PERSIST_ASK_DESTINATION
+					| EMBED_PERSIST_ADD_TO_RECENT);
+
+	galeon_embed_persist_set_fc_parent (persist, window);
+	galeon_embed_persist_set_fc_title (persist, _("Save Page As..."));
+	galeon_embed_persist_set_user_time (persist, gtk_get_current_event_time());
+	galeon_embed_persist_save (persist);
+
+	g_object_unref (persist);
 }
 
 static void 
 embed_popup_save_background_as_cmd (BonoboUIComponent *uic, 
                       	       	    GaleonEmbedPopup *popup, 
                                     const char* verbname)
-{
-	save_property_url (popup, TRUE, FALSE, "background_image");
+{	
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
+	
+	galeon_embed_utils_save_event_property (embed, event, TRUE, FALSE,
+						_("Save Background As..."),
+						"background_image");
 }
 
 static void 
@@ -780,20 +730,11 @@ embed_popup_open_frame_cmd (BonoboUIComponent *uic,
 {
 	char *location;
 	
-	galeon_embed_get_location (popup->priv->embed,
-				   FALSE, FALSE, &location);
+	location = galeon_embed_get_location (popup->priv->embed, FALSE, FALSE);
 
 	galeon_embed_load_url (popup->priv->embed, location);
 }
 
-static void
-embed_popup_reload_frame_cmd (BonoboUIComponent *uic, 
-                      	      GaleonEmbedPopup *popup, 
-                              const char* verbname)
-{
-	/* FIXME implement */
-}
-
 static void 
 embed_popup_open_image_cmd (BonoboUIComponent *uic, 
                       	    GaleonEmbedPopup *popup, 
@@ -804,7 +745,7 @@ embed_popup_open_image_cmd (BonoboUIComponent *uic,
 	const GValue *value;
 	
 	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, "image", &value);
+	value = galeon_embed_event_get_property (info, "image");
 	location = g_value_get_string (value);
 
 	galeon_embed_load_url (popup->priv->embed, location);	
