@@ -61,6 +61,12 @@ typedef struct
 {
         gchar *host;
         gchar *username;
+
+	gchar *password;
+	gchar *httpRealm;
+	gchar *formSubmitURL;
+	gchar *usernameField;
+	gchar *passwordField;
 } PasswordInfo;
 
 typedef struct
