@@ -15,14 +15,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "global-history.h"
 #include "eel-gconf-extensions.h"
 #include "gul-string.h"
 #include "gul-general.h"
-#include "gul-gobject-misc.h"
 #include "galeon-config.h"
 #include "galeon-autocompletion-source.h"
+#include "galeon-auto-bookmarks-source.h"
 #include "galeon-debug.h"
 
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -33,6 +36,7 @@
 #include <glib/gi18n.h>
 
 #include <time.h>
+#include <math.h>
 #include <unistd.h>
 
 /* when parsing using SAX, elements 0 and 1 of the attribute array are
@@ -43,6 +47,10 @@ enum
 	SAX_VALUE = 1
 };
 
+#define GLOBAL_HISTORY_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_GLOBAL_HISTORY, GlobalHistoryPrivate))
+
+
 struct GlobalHistoryPrivate
 {
 	GHashTable *history_sites;
@@ -50,15 +58,23 @@ struct GlobalHistoryPrivate
 	HistoryItem *last_visited_site;
 	gboolean history_dirty;
 	gint expire_days;
+	guint timeout;
 };
 
+/* value for 'toplevel' when unknown */
+#define NOT_SET	((1 << 2)-1)
+
+/* maximum value that fits in 'zoom' */
+#define MAX_ZOOM ((1 << 29)-1)
+
 struct HistoryItemPrivate
 {
 	HistoryItem *host;
-	GHashTable *properties;
+	GHashTable *extra_attributes;
 	GSList *children;
-	gboolean is_host;
-	gint zoom;
+	guint zoom : 29;
+	guint toplevel : 2;	/* tristate: 0, 1, NOT_SET */
+	guint hide : 1;
 };
 
 enum
@@ -69,12 +85,10 @@ enum
         LAST_SIGNAL
 };
 
-static void
-global_history_class_init (GlobalHistoryClass *klass);
 static void
 global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *iface);
 static void
-global_history_init (GlobalHistory *gh);
+global_history_auto_bookmarks_source_init (GaleonAutoBookmarksSourceIface *iface);
 static void
 global_history_finalize (GObject *object);
 static void
@@ -86,32 +100,44 @@ static void
 global_history_autocompletion_source_set_basic_key (GaleonAutocompletionSource *source,
 						    const gchar *basic_key);
 static void
+global_history_auto_bookmarks_source_get_autobookmarks	(GaleonAutoBookmarksSource *source, 
+							 GaleonAutoBookmarksSourceIteratorFunc iterator,
+							 gpointer iterator_data,
+							 GaleonAutoBookmarksSourceFilterFunc filter,
+							 gpointer filter_data,
+							 GaleonAutoBookmarksScoringMethod scoring,
+							 gboolean group_by_host);
+static void
 global_history_emit_changed_data_changed (GlobalHistory *gh);
 
+
 static HistoryItem *
 history_add_host (GlobalHistory *gh, 
 		  const char *name);
 static HistoryItem *
 history_add_item (GlobalHistory *gh,
-		  char *url, 
-		  char *title, 
+		  const char *url, 
+		  const char *title, 
 		  GTime first, 
 		  GTime last, 
-		  gint visits);
+		  gint visits,
+		  guint flags);
 static HistoryItem *
 history_get_host (GlobalHistory *gh, 
 		  const char *url);
-
-static GObjectClass *parent_class = NULL;
+static void
+history_item_free (HistoryItem *hi);
 
 /* how often to save the history, in milliseconds */
 #define HISTORY_SAVE_INTERVAL (60 * 5 * 1000)
 
 static guint global_history_signals[LAST_SIGNAL] = { 0 };
 
-MAKE_GET_TYPE_IFACE (global_history, "GlobalHistory", GlobalHistory,
-		     global_history_class_init, global_history_init, G_TYPE_OBJECT,
-		     global_history_autocompletion_source_init, GALEON_TYPE_AUTOCOMPLETION_SOURCE);
+G_DEFINE_TYPE_WITH_CODE (GlobalHistory, global_history, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GALEON_TYPE_AUTOCOMPLETION_SOURCE,
+						global_history_autocompletion_source_init)
+			 G_IMPLEMENT_INTERFACE (GALEON_TYPE_AUTO_BOOKMARKS_SOURCE,
+						global_history_auto_bookmarks_source_init));
 
 static void
 global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *iface)
@@ -120,13 +146,17 @@ global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *ifac
 	iface->set_basic_key = global_history_autocompletion_source_set_basic_key;
 }
 
+static void
+global_history_auto_bookmarks_source_init (GaleonAutoBookmarksSourceIface *iface)
+{
+	iface->get_autobookmarks = global_history_auto_bookmarks_source_get_autobookmarks;
+}
+
 static void
 global_history_class_init (GlobalHistoryClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize = global_history_finalize;
 
 	global_history_signals[ADD] =
@@ -159,6 +189,8 @@ global_history_class_init (GlobalHistoryClass *klass)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_POINTER);
+
+	g_type_class_add_private (klass, sizeof (GlobalHistoryPrivate));
 }
 
 /** 
@@ -172,6 +204,35 @@ calculate_age (GTime last)
         return days;
 }
 
+static void
+extra_attributes_set (GHashTable **hash, const char *key, const char *value)
+{
+	if (*hash == NULL)
+	{
+		*hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+					       g_free, g_free);
+	}
+
+	g_hash_table_replace (*hash, g_strdup (key), g_strdup (value));
+}
+
+static void
+extra_attributes_free (GHashTable *hash)
+{
+	if (hash != NULL)
+	{
+		g_hash_table_destroy (hash);
+	}
+}
+
+static gboolean
+parse_boolean (const xmlChar *value)
+{
+	return xmlStrcasecmp (value, (const xmlChar*)"true") == 0 ||
+		xmlStrcasecmp (value, (const xmlChar*)"yes") == 0 ||
+		xmlStrEqual (value, (const xmlChar*)"1");
+}	
+
 /**
  * history_parse_host_element: parse <host> elements
  */
@@ -180,37 +241,37 @@ history_parse_host_element (GlobalHistory *gh, const xmlChar **attrs)
 {
 	HistoryItem *host;
 	const xmlChar **attr;
-	gchar *name = NULL;
+	const xmlChar *name = NULL;
 	gint css = -1;
-	gint zoom = -1;
+	gint zoom = 0;
+	GHashTable *extra_attributes = NULL;
 
 	/* parse each attribute */
 	for (attr = attrs; *attr != NULL; attr += 2)
 	{
-		if (xmlStrcmp (attr[SAX_KEY], "name") == 0)
+		if (xmlStrcmp (attr[SAX_KEY], (const xmlChar*)"name") == 0)
 		{
-			name = g_strdup (attr[SAX_VALUE]);
+			name = attr[SAX_VALUE];
 		}
-		else if (xmlStrcmp (attr[SAX_KEY], "authorcss") == 0)
+		else if (xmlStrcmp (attr[SAX_KEY], (const xmlChar*)"authorcss") == 0)
 		{
-			css = atoi (attr[SAX_VALUE]);
+			css = atoi ((char*)attr[SAX_VALUE]);
 		}
-		else if (xmlStrcmp (attr[SAX_KEY], "zoom") == 0)
+		else if (xmlStrcmp (attr[SAX_KEY], (const xmlChar*)"zoom") == 0)
 		{
-			zoom = atoi (attr[SAX_VALUE]);
+			zoom = atoi ((char*)attr[SAX_VALUE]);
 		}
 		else
 		{
-			g_warning ("Unsupported attribute for "
-				   "history.xml::%s: %s", 
-				   attr[SAX_KEY], attr[SAX_VALUE]);
+			extra_attributes_set (&extra_attributes, 
+					      (char*)attr[SAX_KEY], (char*)attr[SAX_VALUE]);
 		}
 	}
 
 	/* create a host structure and store */
-	host = history_add_host (gh, name);
+	host = history_add_host (gh, (const char*)name);
 	host->priv->zoom = zoom;
-	g_free (name);
+	host->priv->extra_attributes = extra_attributes;
 }
 
 /**
@@ -221,40 +282,55 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 {
 	const xmlChar **attr;
 	HistoryItem *item;
-	gchar *title = NULL;
-	gchar *url = NULL;
+	const xmlChar *title = NULL;
+	const xmlChar *url = NULL;
 	GTime first = 0;
 	GTime last = 0;
 	gint visits = 0;
+	gint toplevel = NOT_SET;
+	gboolean hide = FALSE;
+	GHashTable *extra_attributes = NULL;
+	guint flags = 0;
 
 	/* parse each attribute */
 	for (attr = attrs; *attr != NULL; attr += 2)
 	{
-		if (xmlStrcmp(attr[SAX_KEY], "title") == 0)
+		if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"title") == 0)
 		{
-			title = g_strdup (attr[SAX_VALUE]);
+			title = attr[SAX_VALUE];
 		}
-		else if (xmlStrcmp(attr[SAX_KEY], "url") == 0)
+		else if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"url") == 0)
 		{
-			url = g_strdup (attr[SAX_VALUE]);
+			url = attr[SAX_VALUE];
 		}
-		else if (xmlStrcmp(attr[SAX_KEY], "first_time") == 0)
+		else if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"first_time") == 0)
 		{
-			first = strtol (attr[SAX_VALUE], NULL, 10);
+			first = strtol ((char*)attr[SAX_VALUE], NULL, 10);
 		}
-		else if (xmlStrcmp(attr[SAX_KEY], "last_time") == 0)
+		else if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"last_time") == 0)
 		{
-			last = strtol (attr[SAX_VALUE], NULL, 10);
+			last = strtol ((char*)attr[SAX_VALUE], NULL, 10);
 		}
-		else if (xmlStrcmp(attr[SAX_KEY], "visits") == 0)
+		else if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"visits") == 0)
 		{
-			visits = atoi (attr[SAX_VALUE]);
+			visits = atoi ((char*)attr[SAX_VALUE]);
+		}
+		else if (xmlStrcmp(attr[SAX_KEY], (const xmlChar*)"flags") == 0)
+		{
+			flags = atoi ((char*)attr[SAX_VALUE]);
+		}
+		else if (xmlStrcmp (attr[SAX_KEY], (const xmlChar*)"toplevel") == 0)
+		{
+			toplevel = parse_boolean (attr[SAX_VALUE]);
+		}
+		else if (xmlStrcmp (attr[SAX_KEY], (const xmlChar*)"hide") == 0)
+		{
+			hide = parse_boolean (attr[SAX_VALUE]);
 		}
 		else
 		{
-			g_warning("Unsupported attribute for "
-				  "history.xml::%s: %s", 
-				  attr[SAX_KEY], attr[SAX_VALUE]);
+			extra_attributes_set (&extra_attributes,
+					      (char*)attr[SAX_KEY], (char*)attr[SAX_VALUE]);
 		}
 	}
 
@@ -262,26 +338,18 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 	if (calculate_age (last) <= gh->priv->expire_days)
 	{
 		/* add to the history */
-		item = history_add_item (gh, url, title,
-					 first, last, visits);
+		item = history_add_item (gh, (const char*)url, (const char*)title,
+					 first, last, visits, flags);
 
 		g_return_if_fail (item);
 
-		for (attr = attrs; *attr != NULL; attr += 2)
-		{
-			if (xmlStrcmp(attr[SAX_KEY], "prop:") == 0)
-			{
-				global_history_set_page_property (gh,
-								  item,
-								  &attr[SAX_KEY][5],
-								  attr[SAX_VALUE]);
-			}
-		}
+		item->priv->toplevel = toplevel;
+		item->priv->hide = hide;
+		item->priv->extra_attributes = extra_attributes;
 	}
 	else
 	{
-		g_free (url);
-		g_free (title);			
+		extra_attributes_free (extra_attributes);
 	}
 
 }
@@ -298,17 +366,17 @@ history_start_element (void *ctx, const xmlChar *fullname,
 	GlobalHistory    *gh   = ctxt->_private;
 
 	/* parse element */
-	if (xmlStrcmp (fullname, "host") == 0 && attrs != NULL)
+	if (xmlStrcmp (fullname, (const xmlChar*)"host") == 0 && attrs != NULL)
 	{
 		/* <host> element */
 		history_parse_host_element (gh, attrs);
 	}
-	else if (xmlStrcmp (fullname, "item") == 0 && attrs != NULL)
+	else if (xmlStrcmp (fullname, (const xmlChar*)"item") == 0 && attrs != NULL)
 	{
 		/* <item> element */
 		history_parse_item_element (gh, attrs);
 	}
-	else if (xmlStrcmp (fullname, "history") == 0)
+	else if (xmlStrcmp (fullname, (const xmlChar*)"history") == 0)
 	{
 		/* we recurse into this automatically */
 	}
@@ -318,6 +386,37 @@ history_start_element (void *ctx, const xmlChar *fullname,
 	}
 }
 
+/** history_detect_filename: Determine which filename to load history from.
+ *
+ * history2.xml and history.xml are checked in that order, the first file that
+ * exists is returned.  Checking manually if file exists first because libxml
+ * prints warnings to console and doesn't distinguish between file doesn't
+ * exist and parse errors.
+ */
+static char *
+history_detect_filename (void)
+{
+	char *histfile;
+
+	histfile = g_build_filename (g_get_home_dir (), GALEON_DIR, 
+				     "history2.xml", NULL);
+	if (access (histfile, F_OK) == 0)
+	{
+		return histfile;
+	}
+	g_free (histfile);
+
+	histfile = g_build_filename (g_get_home_dir (), GALEON_DIR, 
+				     "history.xml", NULL);
+	if (access (histfile, F_OK) == 0)
+	{
+		return histfile;
+	}
+	g_free (histfile);
+
+	return NULL;
+}
+
 /**
  * history_load: Loads the history from the history file (if it
  * exists). Creates the history dialog. Does not load expired entries
@@ -355,15 +454,13 @@ history_load (GlobalHistory *gh)
 	};
         gchar *histfile;
 
-	/* build the filename */
-	histfile = g_build_filename (g_get_home_dir (), GALEON_DIR, 
-				     "history.xml", NULL);
-	
 	/* build the initial history hash table */
-        gh->priv->history_sites = g_hash_table_new (g_str_hash, g_str_equal);
+        gh->priv->history_sites = g_hash_table_new_full (g_str_hash, g_str_equal,
+							 NULL, (GDestroyNotify)history_item_free);
 
 	/* build the initial history hosts hashtable */
-	gh->priv->history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
+	gh->priv->history_hosts = g_hash_table_new_full (g_str_hash, g_str_equal,
+							 NULL, (GDestroyNotify)history_item_free);
 
 	/* find out how many days to keep before expiring elements */
 	gh->priv->expire_days = eel_gconf_get_integer (CONF_HISTORY_EXPIRE);
@@ -371,24 +468,22 @@ history_load (GlobalHistory *gh)
 	LOG ("Loading history.");
 	START_PROFILER ("Loading History");
 
-	/* load the file if we can access it */
-	if (access (histfile, F_OK) != -1)
+	histfile = history_detect_filename ();
+	if (histfile != NULL)
 	{
 		xmlSubstituteEntitiesDefault (1);
 		xmlSAXParseFileWithData (&parser, histfile, TRUE, gh);
-	}
+
+		g_free (histfile);
+	}	
 
 	STOP_PROFILER ("Loading History");
-	
-	/* free allocated strings */
-	g_free (histfile);
 
 	LOG ("History loaded: %d sites, %d hosts.", 
 	     g_hash_table_size (gh->priv->history_sites),
 	     g_hash_table_size (gh->priv->history_hosts));
 }
 
-
 static void
 history_save_item_property (gpointer  key,
 			    gpointer  value,
@@ -396,13 +491,7 @@ history_save_item_property (gpointer  key,
 {
 	xmlTextWriterPtr writer = (xmlTextWriterPtr)user_data;
 
-	g_return_if_fail (key);
-	g_return_if_fail (value);
-
-	/* I'm not convinced that the person that wrote this code
-	 * originally meant to put the "prop:" bit before the value
-	 * - crispin 30/03/2004 */
-	xmlTextWriterWriteFormatAttribute (writer, (gchar*)key, "prop:%s", (gchar*)value);
+	xmlTextWriterWriteAttribute (writer, (const xmlChar*)key, (const xmlChar*)value);
 }
 
 /**
@@ -418,17 +507,21 @@ history_save_host (gpointer key, gpointer value, gpointer user_data)
 
 	if (!hh->priv->children) return;
 
-	xmlTextWriterStartElement (writer, "host");
-	xmlTextWriterWriteAttribute (writer, "name", hh->url);
+	xmlTextWriterStartElement (writer, (const xmlChar*)"host");
+	xmlTextWriterWriteAttribute (writer, (const xmlChar*)"name", (const xmlChar*)hh->url);
 
 	if (hh->priv->zoom > 0)
 	{
-		xmlTextWriterWriteFormatAttribute (writer, "zoom", "%d", hh->priv->zoom);
+		xmlTextWriterWriteFormatAttribute (writer, 
+                          (const xmlChar*)"zoom", "%d", hh->priv->zoom);
 	}
 
-	g_hash_table_foreach (hh->priv->properties,
-			      (GHFunc)history_save_item_property,
-			      writer);
+	if (hh->priv->extra_attributes != NULL)
+	{
+		g_hash_table_foreach (hh->priv->extra_attributes,
+				      (GHFunc)history_save_item_property,
+				      writer);
+	}
 	
 	xmlTextWriterEndElement (writer);
 }
@@ -445,18 +538,29 @@ history_save_item (gpointer key, gpointer value, gpointer user_data)
 	g_return_if_fail (hi->title);
 	g_return_if_fail (hi->url);
 
-	xmlTextWriterStartElement (writer, "item");
+	xmlTextWriterStartElement (writer, (const xmlChar*)"item");
 
-	xmlTextWriterWriteAttribute (writer, "title", hi->title);
-	xmlTextWriterWriteAttribute (writer, "url", hi->url);
+	xmlTextWriterWriteAttribute (writer, (const xmlChar*)"title", (const xmlChar*)hi->title);
+	xmlTextWriterWriteAttribute (writer, (const xmlChar*)"url", (const xmlChar*)hi->url);
 
-	xmlTextWriterWriteFormatAttribute (writer, "first_time", "%d", hi->first);
-	xmlTextWriterWriteFormatAttribute (writer, "last_time", "%d", hi->last);
-	xmlTextWriterWriteFormatAttribute (writer, "visits", "%d", hi->visits);
-
-	g_hash_table_foreach (hi->priv->properties,
-			      (GHFunc)history_save_item_property,
-			      writer);
+	xmlTextWriterWriteFormatAttribute (writer, (const xmlChar*)"first_time", "%d", hi->first);
+	xmlTextWriterWriteFormatAttribute (writer, (const xmlChar*)"last_time", "%d", hi->last);
+	xmlTextWriterWriteFormatAttribute (writer, (const xmlChar*)"visits", "%d", hi->visits);
+	xmlTextWriterWriteFormatAttribute (writer, (const xmlChar*)"flags", "%d", hi->flags);
+	if (hi->priv->toplevel != NOT_SET)
+	{
+		xmlTextWriterWriteAttribute (writer, (const xmlChar*)"toplevel", (const xmlChar*)(hi->priv->toplevel ? "yes" : "no"));
+	}
+	if (hi->priv->hide)
+	{
+		xmlTextWriterWriteAttribute (writer, (const xmlChar*)"hide", (const xmlChar*)"yes");
+	}
+	if (hi->priv->extra_attributes)
+	{
+		g_hash_table_foreach (hi->priv->extra_attributes,
+				      (GHFunc)history_save_item_property,
+				      writer);
+	}
 	
 	xmlTextWriterEndElement (writer);
 }
@@ -472,7 +576,7 @@ history_save_to_file (GlobalHistoryPrivate *ghpriv, xmlTextWriterPtr writer)
 
 	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
 	if (ret < 0) return ret;
-	ret = xmlTextWriterStartElement (writer, "history");
+	ret = xmlTextWriterStartElement (writer, (const xmlChar*)"history");
 	if (ret < 0) return ret;
 
         g_hash_table_foreach (ghpriv->history_hosts, history_save_host, writer);
@@ -506,13 +610,13 @@ history_save (GlobalHistoryPrivate *ghpriv)
 
 	histfile = g_build_filename (g_get_home_dir (),
 				     GALEON_DIR,
-				     "history.xml",
+				     "history2.xml",
 				     NULL);
 	tmpfile = g_strconcat (histfile, ".tmp", NULL);
 
 	writer = xmlNewTextWriterFilename (tmpfile, 0);
 	xmlTextWriterSetIndent (writer, 1);
-	xmlTextWriterSetIndentString (writer, "  ");
+	xmlTextWriterSetIndentString (writer, (const xmlChar*)"  ");
 
 	START_PROFILER ("Saving History");
 
@@ -522,7 +626,7 @@ history_save (GlobalHistoryPrivate *ghpriv)
 
 	if (ret < 0)
 	{
-		g_warning( "Failed writing to history.xml" );
+		g_warning( "Failed writing to history2.xml" );
 	}
 	else
 	{
@@ -558,33 +662,37 @@ history_periodic_save_cb (GlobalHistoryPrivate *ghpriv)
 }
 
 /**
- * history_item_free_cb: free one history item
+ * history_item_free: free one history item
  */
 static void
-history_item_free_cb (gpointer key, gpointer value, GlobalHistory *gh)
+history_item_free (HistoryItem *hi)
 {
-	HistoryItem *hi = (HistoryItem *)value;
-
-	if (!hi->priv->is_host)
+	if (hi->priv->host)
 	{
 		HistoryItem *host = hi->priv->host;
-		if (host) /* I think host can't be NULL, but I'm not sure */
-		{
-			host->priv->children = g_slist_remove (host->priv->children, hi);
-		}
+		host->priv->children = g_slist_remove (host->priv->children, hi);
 	}
 	
-	/* if this was the last visited item, make it NULL. 
-	 * It will be recalculated when needed */
-	if (hi == gh->priv->last_visited_site)
-	{
-		gh->priv->last_visited_site = NULL;
-	}
-
 	g_free (hi->url); 
 	g_free (hi->title);
 
-	g_hash_table_destroy (hi->priv->properties);
+	extra_attributes_free (hi->priv->extra_attributes);
+
+	g_free (hi->priv);
+	g_free (hi);
+}
+
+static gboolean
+history_item_should_show (const HistoryItem *item)
+{
+	g_return_val_if_fail (item != NULL, TRUE);
+
+	/* Only show URLs loaded in toplevel window (not frames(?))
+	 * Since this information wasn't originally saved in the history we'll
+	 * also include the URLs that don't have the flag specifically set
+	 * (otherwise the history dialog would suddenly show up empty)
+	 */
+	return !item->priv->hide && item->priv->toplevel != FALSE;
 }
 
 static void
@@ -601,32 +709,28 @@ global_history_finalize (GObject *object)
 
 	/* save the history */
 	history_save (gh->priv);
-	
+
+	g_source_remove (gh->priv->timeout);
+
 	/* destroy hosts hash table */
-	g_hash_table_foreach_remove (gh->priv->history_sites, (GHRFunc)
-				     history_item_free_cb, gh);
 	g_hash_table_destroy (gh->priv->history_sites);
 	gh->priv->history_sites = NULL;
 
 	/* destroy sites hash table */
-	g_hash_table_foreach_remove (gh->priv->history_hosts, 
-				     (GHRFunc)history_item_free_cb,
-				     gh);
 	g_hash_table_destroy (gh->priv->history_hosts);
 	gh->priv->history_hosts = NULL;
 	gh->priv->last_visited_site = NULL;
 
         g_return_if_fail (gh->priv != NULL);
 
-        g_free (gh->priv);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (global_history_parent_class)->finalize (object);
 }
 
 static void
 global_history_init (GlobalHistory *gh)
 {
-        gh->priv = g_new0 (GlobalHistoryPrivate, 1);
+        gh->priv = GLOBAL_HISTORY_GET_PRIVATE (gh);
 	gh->priv->history_sites = NULL;
 	gh->priv->history_hosts = NULL;
 	gh->priv->history_dirty = FALSE;
@@ -636,16 +740,16 @@ global_history_init (GlobalHistory *gh)
 	history_load (gh);
 
 	/* setup the periodic history saving callback */
-	g_timeout_add (HISTORY_SAVE_INTERVAL, 
-		       (GSourceFunc)history_periodic_save_cb, 
-		       gh->priv);
+	gh->priv->timeout = g_timeout_add (HISTORY_SAVE_INTERVAL, 
+					   (GSourceFunc)history_periodic_save_cb, 
+					   gh->priv);
 }
 
 GlobalHistory *
 global_history_new (void)
 {
 	return GLOBAL_HISTORY (g_object_new 
-			       (GLOBAL_HISTORY_TYPE, NULL));
+			       (TYPE_GLOBAL_HISTORY, NULL));
 }
 
 static void
@@ -664,24 +768,33 @@ history_add_host (GlobalHistory *gh,
 		  const char *name)
 {
 	HistoryItem *host;
+	const char *title;
 	
 	g_return_val_if_fail (name, NULL);
 
+	if (g_str_equal (name, "Local files"))
+	{
+		title = _("Local files");
+	}
+	else if (g_str_equal (name, "Other"))
+	{
+		title = _("Other");
+	}
+	else
+	{
+		title = name;
+	}
+
 	/* allocate */
 	host = g_new0 (HistoryItem, 1);
 
 	/* fill in structure */
-	host->title = g_strdup (name);
+	host->title = g_strdup (title);
 	host->url = g_strdup (name);
-	host->first = 0;
-	host->last = 0;
-	host->visits = 0;
 	
 	host->priv = g_new0 (HistoryItemPrivate, 1);
-	host->priv->children = NULL;
-	host->priv->properties = g_hash_table_new (g_str_hash, 
-						   g_str_equal);
-	
+	host->priv->toplevel = NOT_SET;
+
 	/* add to the hosts hashtable */
 	g_hash_table_insert (gh->priv->history_hosts, host->url, host);
 
@@ -703,29 +816,24 @@ history_get_host (GlobalHistory *gh,
 	/* check args */
 	g_return_val_if_fail (url, NULL);
 
- 	/* check if it's a local file */
- 	if (!g_ascii_strncasecmp (url, "file://", 7))
- 	{
- 		/* if so, put it in a folder named "Local files" */
- 		name = _("Local files");
-  	}
-	else
+	vfs_uri = gnome_vfs_uri_new (url);
+	if (vfs_uri != NULL)
 	{
-		/* parse the url as a GnomeVFS uri */
-		vfs_uri = gnome_vfs_uri_new (url);
-		if (vfs_uri != NULL)
-		{
-			name = gnome_vfs_uri_get_host_name (vfs_uri);
-		}
+		name = gnome_vfs_uri_get_host_name (vfs_uri);
 
-		/* handle failure gracefully */
-		if (name == NULL)
+		/* can't free URI yet as it owns the hostname */
+
+		if (name == NULL && gnome_vfs_uri_is_local (vfs_uri))
 		{
-			/* FIXME gnome vfs doesnt handle https correctly */
-			name = _("Other");
+			name = "Local files";
 		}
 	}
 
+	if (name == NULL)
+	{
+		name = "Other";
+	}
+
 	/* lookup in table */
 	host = g_hash_table_lookup (gh->priv->history_hosts, name);
 	if (host == NULL)
@@ -734,8 +842,6 @@ history_get_host (GlobalHistory *gh,
 		host = history_add_host (gh, name);
 	}
 
-	/* free uri -- this must be done here since the hostname returned
-	 * is a constant string and part of the vfs_uri structure */
 	if (vfs_uri != NULL)
 	{
 		gnome_vfs_uri_unref (vfs_uri);
@@ -750,11 +856,12 @@ history_get_host (GlobalHistory *gh,
  */
 static HistoryItem *
 history_add_item (GlobalHistory *gh,
-		  char *url, 
-		  char *title, 
+		  const char *url, 
+		  const char *title, 
 		  GTime first, 
 		  GTime last, 
-		  gint visits)
+		  gint visits,
+		  guint flags)
 {
 	HistoryItem *hi;
 	HistoryItem *host;
@@ -771,24 +878,30 @@ history_add_item (GlobalHistory *gh,
 	g_return_val_if_fail (host, NULL);
 
 	/* fill in the fields */
-	hi->url = url;
-	hi->title = title;
+	hi->url = g_strdup (url);
+	hi->title = g_strdup (title);
 	hi->first = first;
 	hi->last = last;
 	hi->visits = visits;
+	hi->flags = flags;
 	hi->priv = g_new0 (HistoryItemPrivate, 1);
-	hi->priv->children = NULL;
 	hi->priv->host = host;
-	hi->priv->properties = g_hash_table_new (g_str_hash, 
-						 g_str_equal);
+	hi->priv->toplevel = NOT_SET;
 	
 	/* update the host */
 	host->visits += visits;
 
 	/* first item of host */
-	host->first = first;
-	host->last = last;
-	host->priv->children = g_slist_append (host->priv->children, hi);
+	if (!host->first || first < host->first)
+	{
+		host->first = first;
+	}
+	
+	if (last > host->last)
+	{
+		host->last = last;
+	}
+	host->priv->children = g_slist_prepend (host->priv->children, hi);
 
 	/* add to the table */
         g_hash_table_insert (gh->priv->history_sites, 
@@ -805,7 +918,9 @@ history_add_item (GlobalHistory *gh,
 
 void     
 global_history_visited (GlobalHistory *gh,
-			const char *url)
+			const char *url,
+			gboolean redirect,
+			gboolean toplevel)
 {
         HistoryItem *hi;
         GTime now;
@@ -813,6 +928,10 @@ global_history_visited (GlobalHistory *gh,
 	/* check arguments */
 	g_return_if_fail (url != NULL);
 
+	/* canonicalize for bitfields */
+	redirect = redirect != FALSE;
+	toplevel = toplevel != FALSE;
+
 	/* get current time */
         now = time (NULL);
 
@@ -824,6 +943,21 @@ global_history_visited (GlobalHistory *gh,
                 hi->last = now;
                 hi->visits++;
 
+		/* hide and redirect seem equivalent, so not bothering to save
+		 * the bits individually here
+		 */
+		hi->priv->hide = redirect;
+
+		/* if an URL is ever seen as 'toplevel' never downgrade it and
+		 * make it disappear from the history dialog.  Consider:
+		 *   open frameset: frame == !toplevel
+		 *   open frame in tab: frame == toplevel
+		 */
+		if (hi->priv->toplevel != TRUE)
+		{
+			hi->priv->toplevel = toplevel;
+		}
+
 		g_return_if_fail (hi->priv->host != NULL);
 		
 		/* update host data as well */
@@ -834,13 +968,12 @@ global_history_visited (GlobalHistory *gh,
 	else
 	{
 		/* create a new item to add to the view */
-		hi = history_add_item (gh,
-				       g_strdup (url), 
-				       g_strdup (_("Untitled")),
-				       now, now, 1);
+		hi = history_add_item (gh, url, "", now, now, 1, 0);
 
 		if (hi)
 		{
+			hi->priv->hide = redirect;
+			hi->priv->toplevel = toplevel;
 			g_signal_emit (G_OBJECT (gh), global_history_signals[ADD], 0, hi);
 		}
         }
@@ -933,6 +1066,45 @@ global_history_get_page_title (GlobalHistory *gh,
 	else return NULL;
 }
 
+gboolean
+global_history_set_page_flags (GlobalHistory *gh,
+                               const char *url,
+                               guint flags)
+{
+	HistoryItem *hi;
+
+	/* lookup in history */
+        hi = g_hash_table_lookup (gh->priv->history_sites, url);
+
+	/* check its there */
+	if (hi == NULL)
+	{
+		/* FIXME: this certainly happens, should we add the
+		 * URL to the history at this point? I don't know -MattA */
+		return FALSE;
+	}
+
+	hi->flags = flags;
+
+	/* we have changed the history, so save at next checkpoint */
+	gh->priv->history_dirty = TRUE;
+
+	g_signal_emit (G_OBJECT (gh), global_history_signals[UPDATE], 0, hi);
+	global_history_emit_changed_data_changed (gh);
+	
+	return TRUE;
+}
+
+guint
+global_history_get_page_flags (GlobalHistory *gh,
+                               const char *url)
+{
+	HistoryItem *hi;
+
+	hi = g_hash_table_lookup (gh->priv->history_sites, url);
+
+	return hi ? hi->flags : (guint)-1;
+}
 
 gboolean 
 global_history_set_host_zoom (GlobalHistory *gh,
@@ -951,7 +1123,7 @@ global_history_set_host_zoom (GlobalHistory *gh,
 
 	hh = hi->priv->host;
 
-	hh->priv->zoom = zoom;
+	hh->priv->zoom = CLAMP(zoom, 0, MAX_ZOOM);
 
 	/* we have changed the history, so save at next checkpoint */
 	gh->priv->history_dirty = TRUE;
@@ -981,10 +1153,9 @@ global_history_get_host_zoom (GlobalHistory *gh,
 
 	hh = hi->priv->host;
 
-	return hh->priv->zoom;
+	return (hh->priv->zoom > 0) ? hh->priv->zoom : -1;
 }
 
-
 gboolean 
 global_history_remove_url (GlobalHistory *gh,
 			   const char *url)
@@ -999,8 +1170,16 @@ global_history_remove_url (GlobalHistory *gh,
 
 	if (hi)
 	{	
+		/* if this was the last visited item, make it NULL.
+		 * It will be recalculated when needed */
+		if (hi == gh->priv->last_visited_site)
+		{
+			gh->priv->last_visited_site = NULL;
+		}
+
 		g_signal_emit (G_OBJECT (gh), global_history_signals[REMOVE], 0, hi);
-		history_item_free_cb ((gpointer)url, hi, gh);
+		g_hash_table_remove (gh->priv->history_sites, url);
+
 		global_history_emit_changed_data_changed (gh);
 		gh->priv->history_dirty = TRUE;
 		return FALSE;
@@ -1019,8 +1198,10 @@ global_history_clear (GlobalHistory *gh)
 	/* no last_visited_site */
 	p->last_visited_site = NULL;
 
-	p->history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
-	p->history_sites = g_hash_table_new (g_str_hash, g_str_equal);
+	p->history_hosts = g_hash_table_new_full (g_str_hash, g_str_equal,
+						  NULL, (GDestroyNotify)history_item_free);
+	p->history_sites = g_hash_table_new_full (g_str_hash, g_str_equal,
+						  NULL, (GDestroyNotify)history_item_free);
 
 	global_history_emit_changed_data_changed (gh);
 
@@ -1029,17 +1210,10 @@ global_history_clear (GlobalHistory *gh)
 	history_save (gh->priv);
 
 	/* Destroy old history hash table */
-	g_hash_table_foreach (old_history_sites, 
-			      (GHFunc)history_item_free_cb, 
-			      gh);
 	g_hash_table_destroy (old_history_sites);
 
 	/* destroy old hosts hash table */
-	g_hash_table_foreach (old_history_hosts, 
-			      (GHFunc)history_item_free_cb, 
-			      gh);
 	g_hash_table_destroy (old_history_hosts);
-
 }
 
 /**
@@ -1066,24 +1240,24 @@ global_history_get_host_list (GlobalHistory *gh)
 	return list;
 }
 
-void 
-global_history_set_page_property  (GlobalHistory *gh,
-				   HistoryItem *hi,
-				   const char *key,
-				   const char *value)
+static void 
+history_add_item_to_slist (gpointer key, HistoryItem *item, GSList **list)
 {
-	g_hash_table_replace (hi->priv->properties,
-			      (gpointer)g_strdup(key),
-			      (gpointer)g_strdup(value));
+	if (history_item_should_show (item))
+	{
+		*list = g_slist_prepend (*list, item);
+	}
 }
 
-char * 
-global_history_get_page_property  (GlobalHistory *gh,
-				   HistoryItem *hi,
-				   const gchar *key)
+static GSList *
+global_history_get_sites_slist (GlobalHistory *gh)
 {
-	return g_hash_table_lookup (hi->priv->properties,
-				    (gpointer)key);
+	GSList *list = NULL;
+
+	g_hash_table_foreach (gh->priv->history_sites, 
+			      (GHFunc) history_add_item_to_slist, &list);
+
+	return list;
 }
 
 static gboolean
@@ -1101,10 +1275,15 @@ filter_by_date (int filter_type,
         gboolean result;
 
         g_date_clear (&current_date, 1);
-        g_date_set_time (&current_date, time (NULL));
-
         g_date_clear (&date, 1);
+
+#if GLIB_CHECK_VERSION (2,9,0)
+        g_date_set_time_t (&current_date, time (NULL));
+        g_date_set_time_t (&date, atime);
+#else
+        g_date_set_time (&current_date, time (NULL));
         g_date_set_time (&date, atime);
+#endif
 
         switch (filter_type)
         {
@@ -1144,9 +1323,11 @@ history_add_url_to_list (HistoryItem *item,
 			 HistoryFilter *filter,
 			 GList **list)
 {
-	gboolean add = TRUE;
+	gboolean add;
+
+	add = history_item_should_show (item);
 	
-	if (filter != NULL)
+	if (add && filter != NULL)
 	{
 		add = ((filter_by_date (filter->type, item->last)) &&
 		       (filter_by_word (item->url, filter->word)));
@@ -1191,6 +1372,12 @@ global_history_autocompletion_source_foreach_aux (gpointer key, gpointer value,
 {
         HistoryItem *hi = (HistoryItem *) value;
 	gpointer *little_hack = user_data;
+
+	if (!history_item_should_show (hi))
+	{
+		return;
+	}
+
 	((GaleonAutocompletionSourceForeachFunc) little_hack[1]) 
 		(little_hack[0], hi->url, hi->title, 
 		 /* very simple scoring function. Most recent entries get higher scores and each
@@ -1224,3 +1411,126 @@ global_history_emit_changed_data_changed (GlobalHistory *gh)
 	g_signal_emit_by_name (gh, "data-changed");
 }
 
+/* Autobookmarks */
+
+static int 
+autobm_score_both (const HistoryItem *item)
+{
+	return item->last + (item->visits << 17);	
+}
+
+static gint 
+autobm_compare_both (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = autobm_score_both (a);
+	int score_b = autobm_score_both (b);
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static gint 
+autobm_compare_recently_visited (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = a->last;
+	int score_b = b->last;
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static gint 
+autobm_compare_frequently_visited (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = a->visits;
+	int score_b = b->visits;
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static HistoryItem *
+global_history_get_host_dominant_item (const HistoryItem *host, 
+				       GaleonAutoBookmarksSourceFilterFunc filter, gpointer filter_data, 
+				       GCompareFunc compare_func)
+{
+	const GSList *it;
+	host->priv->children = g_slist_sort (host->priv->children, compare_func);
+
+	for (it = host->priv->children; it; it = it->next)
+	{
+		HistoryItem *ret = it->data;
+
+		if (history_item_should_show (ret) &&
+		    filter (ret->title, ret->url, filter_data))
+		{
+			return ret;
+		}
+	}
+	return NULL;
+}
+
+static void
+global_history_auto_bookmarks_source_get_autobookmarks	(GaleonAutoBookmarksSource *source, 
+							 GaleonAutoBookmarksSourceIteratorFunc iterator,
+							 gpointer iterator_data,
+							 GaleonAutoBookmarksSourceFilterFunc filter,
+							 gpointer filter_data,
+							 GaleonAutoBookmarksScoringMethod scoring,
+							 gboolean group_by_host)
+{
+	GlobalHistory *gh = GLOBAL_HISTORY (source);
+	GList *host_list;
+	const GList *it;
+	GSList *candidates = NULL;
+	const GSList *sit;
+	gboolean cont;
+	GCompareFunc compare_func = 
+		scoring == GALEON_AUTO_BOOKMARKS_SCORING_RECENTLY_VISITED ? (GCompareFunc) autobm_compare_recently_visited
+		: scoring == GALEON_AUTO_BOOKMARKS_SCORING_FRECUENTLY_VISITED ? (GCompareFunc) autobm_compare_frequently_visited
+		: (GCompareFunc) autobm_compare_both;
+
+	START_PROFILER ("history_generate_autobookmarks");
+
+	if (group_by_host)
+	{
+	     START_PROFILER ("history_generate_autobookmarks updating dominant_item");
+	     host_list = global_history_get_host_list (gh);
+	     for (it = host_list; it; it = it->next)
+	     {
+		  HistoryItem *host = it->data;
+		  HistoryItem *di = global_history_get_host_dominant_item (host, filter, filter_data, compare_func);
+		  if (di)
+		  {
+			  candidates = g_slist_prepend (candidates, di);
+		  }
+	     }
+	     g_list_free (host_list);
+	     STOP_PROFILER ("history_generate_autobookmarks updating dominant_item");
+	}
+	else
+	{
+	     START_PROFILER ("history_generate_autobookmarks getting list");
+	     candidates = global_history_get_sites_slist (gh);
+	     STOP_PROFILER ("history_generate_autobookmarks getting list");
+	}
+
+	START_PROFILER ("history_generate_autobookmarks sorting");
+	candidates = g_slist_sort (candidates, compare_func);
+	STOP_PROFILER ("history_generate_autobookmarks sorting");
+
+	cont = TRUE;
+	for (sit = candidates; sit && cont; sit = sit->next)
+	{
+		HistoryItem *hi = sit->data;
+
+		if (group_by_host /* already filtered */
+		    || filter (hi->title, hi->url, filter_data))
+		{
+			/* g_print ("AB: %s - %s\n", hi->title, hi->url); */
+			cont = iterator (hi->title, hi->url, iterator_data);
+		}
+	}
+
+	g_slist_free (candidates);
+
+	STOP_PROFILER ("history_generate_autobookmarks");
+}
+
