@@ -665,15 +665,20 @@ print_printer_combo_cb (GtkWidget *widget,
 	GtkTreeModel *model;
 	GPtrArray *paper_array;
 	guint i;
+	gint combo_selection;
 
 	name_combo = galeon_dialog_get_control(dialog, NAME_PROP);
 	command_entry = galeon_dialog_get_control(dialog, COMMAND_PROP);
 
-	priv->selected_printer =
-			gtk_combo_box_get_active(GTK_COMBO_BOX(name_combo));
+	/* Ignore callback if combo box was not yet initialized */
+	combo_selection = gtk_combo_box_get_active(GTK_COMBO_BOX(name_combo));
+	if (combo_selection < 0)
+		return;
 
+	priv->selected_printer = combo_selection;
 	info = (EmbedPrintInfo *)g_list_nth_data(priv->printers,
 	                                         priv->selected_printer);
+	g_return_if_fail (info != NULL);
 
 	gtk_widget_set_sensitive(command_entry, info->can_change_command);
 
