@@ -849,6 +849,21 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	return MozillaPrivate::LoadOverrideStyleSheet (docShell, uri, aStyleSheet);
 }
 
+#ifdef HAVE_NSISTYLESHEETSERVICE_H
+nsresult GaleonWrapper::RemoveOverrideStyleSheet(char *css)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIURI> uri;
+	rv = GaleonUtils::NewURI(getter_AddRefs(uri), GulCString (css));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	nsCOMPtr<nsIDocShell> docShell = do_GetInterface (mWebBrowser);
+	NS_ENSURE_TRUE (docShell, NS_ERROR_FAILURE);
+
+	return MozillaPrivate::RemoveOverrideStyleSheet (docShell, uri);
+}
+#else
 nsresult GaleonWrapper::RemoveOverrideStyleSheet(nsIStyleSheet *remove)
 {
 	nsCOMPtr<nsIDocShell> docShell = do_GetInterface (mWebBrowser);
@@ -856,6 +871,7 @@ nsresult GaleonWrapper::RemoveOverrideStyleSheet(nsIStyleSheet *remove)
 
 	return MozillaPrivate::RemoveOverrideStyleSheet (docShell, remove);
 }
+#endif
 
 nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
 {
