@@ -31,11 +31,7 @@
  *
  */
 
-#ifdef HAVE_NSIGLOBALHISTORY3_H
 NS_IMPL_ISUPPORTS2(MozGlobalHistory, nsIGlobalHistory2, nsIGlobalHistory3)
-#else
-NS_IMPL_ISUPPORTS1(MozGlobalHistory, nsIGlobalHistory2)
-#endif
 
 MozGlobalHistory::MozGlobalHistory ()
 {
@@ -49,12 +45,8 @@ MozGlobalHistory::~MozGlobalHistory ()
 /* void addURI (in nsIURI aURI, in boolean aRedirect, in boolean aToplevel); */
 NS_IMETHODIMP MozGlobalHistory::AddURI (nsIURI *aURI,
 					PRBool aRedirect,
-					PRBool aTopLevel
-#ifdef HAVE_NSIBROWSERHISTORY_ADDURI_WITH_REFERRER
-					,nsIURI *aReferrer
-#endif
-					
-					 )
+					PRBool aTopLevel,
+					nsIURI *aReferrer)
 {
 	nsresult rv;
 	NS_ENSURE_ARG_POINTER(aURI);
@@ -150,7 +142,6 @@ NS_IMETHODIMP MozGlobalHistory::SetURIGeckoFlags(nsIURI *aURI,
 }
 #endif
 
-#ifdef HAVE_NSIGLOBALHISTORY3_H
 /* void addDocumentRedirect (in nsIChannel aOldChannel, in nsIChannel aNewChannel, in PRInt32 aFlags, in boolean aTopLevel); */
 NS_IMETHODIMP MozGlobalHistory::AddDocumentRedirect(nsIChannel *aOldChannel,
                                                     nsIChannel *aNewChannel,
@@ -159,4 +150,3 @@ NS_IMETHODIMP MozGlobalHistory::AddDocumentRedirect(nsIChannel *aOldChannel,
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
-#endif
