@@ -23,11 +23,7 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_NSIGLOBALHISTORY3_H
 #include <nsIGlobalHistory3.h>
-#else
-#include <nsIGlobalHistory2.h>
-#endif
 
 #include "global-history.h"
 
@@ -37,11 +33,7 @@
  { 0xbe0c42c1, 0x39d4, 0x4271, { 0xb7, 0x9e, 0xf7, 0xaa, 0x49, 0xeb, 0x6a, 0x15}}
 
 class MozGlobalHistory:
-#ifdef HAVE_NSIGLOBALHISTORY3_H
 	public nsIGlobalHistory3
-#else
-	public nsIGlobalHistory2
-#endif
 {
 	public:
 		MozGlobalHistory ();
@@ -49,9 +41,7 @@ class MozGlobalHistory:
 
 		NS_DECL_ISUPPORTS
 		NS_DECL_NSIGLOBALHISTORY2
-#ifdef HAVE_NSIGLOBALHISTORY3_H
 		NS_DECL_NSIGLOBALHISTORY3
-#endif
 
 	private:
 		GlobalHistory *mGlobalHistory;
