@@ -41,6 +41,11 @@
 #include "GtkNSSSecurityWarningDialogs.h"
 #endif
 
+#ifdef HAVE_NSIXULAPPINFO_H
+#include <nsXULAppAPI.h>
+#include "EphyXULAppInfo.h"
+#endif
+
 #include "ExternalProtocolService.h"
 #include "GaleonAboutModule.h"
 
@@ -85,6 +90,9 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
 #ifdef HAVE_NSIJSCONSOLESERVICE_H
 NS_GENERIC_FACTORY_CONSTRUCTOR(JSConsoleService)
 #endif
+#ifdef HAVE_NSIXULAPPINFO_H
+NS_GENERIC_FACTORY_CONSTRUCTOR(EphyXULAppInfo)
+#endif
 
 // NS_DOWNLOAD_CONTRACTID was renamed in mozilla 1.8b
 #ifndef NS_TRANSFER_CONTRACTID
@@ -251,6 +259,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		EphyPromptServiceConstructor
 	},
 #endif /* HAVE_NSINONBLOCKINGALERTSERVICE_H */
+#ifdef HAVE_NSIXULAPPINFO_H
+	{
+		EPHY_XUL_APP_INFO_CLASSNAME,
+		EPHY_XUL_APP_INFO_CID,
+		XULAPPINFO_SERVICE_CONTRACTID,
+		EphyXULAppInfoConstructor
+	},
+#endif
 };
 
 #define NS_MSGCONTENTPOLICY_CONTRACTID "@mozilla.org/messenger/content-policy;1"
