@@ -56,6 +56,9 @@
 #include <nsIDOMWindow.h>
 #include <nsIDOMNSDocument.h>
 #include <nsCOMPtr.h>
+#ifdef HAVE_NSISTYLESHEETSERVICE_H
+#include <nsIStyleSheetService.h>
+#endif
 
 #include "GaleonUtils.h"
 
@@ -155,6 +158,27 @@ MozillaPrivate::LoadOverrideStyleSheet (nsIDocShell *aDocShell, nsIURI *aUri,
 {
 	nsresult rv;
 
+#ifdef HAVE_NSISTYLESHEETSERVICE_H
+	nsCOMPtr<nsIStyleSheetService> service
+			(do_GetService ("@mozilla.org/content/style-sheet-service;1", &rv));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	PRBool isRegistered = PR_FALSE;
+	rv = service->SheetRegistered (aUri, nsIStyleSheetService::USER_SHEET,
+				       &isRegistered);
+	if (NS_SUCCEEDED (rv) && isRegistered)
+	{
+		rv = service->UnregisterSheet (aUri, nsIStyleSheetService::USER_SHEET);
+	}
+
+	rv = service->LoadAndRegisterSheet (aUri, nsIStyleSheetService::AGENT_SHEET);
+	if (NS_FAILED (rv))
+	{
+		g_warning ("Registering the user stylesheet failed (rv=%x)!\n", rv);
+	}
+
+	return rv;
+#else
 	nsCOMPtr<nsIPresShell> presShell;
 	aDocShell->GetPresShell (getter_AddRefs(presShell));
 	NS_ENSURE_TRUE (presShell, NS_ERROR_FAILURE);
@@ -165,7 +189,6 @@ MozillaPrivate::LoadOverrideStyleSheet (nsIDocShell *aDocShell, nsIURI *aUri,
 	nsICSSLoader *loader = doc->CSSLoader();
 	NS_ENSURE_TRUE (loader, NS_ERROR_FAILURE);
 
-#ifdef HAVE_NSSTRING_INTERNAL
 	/* load sheet */
 	nsCOMPtr<nsICSSStyleSheet> sheet;
 #ifdef HAVE_NSICSSLOADER_LOADSHEETSYNC
@@ -194,15 +217,38 @@ MozillaPrivate::LoadOverrideStyleSheet (nsIDocShell *aDocShell, nsIURI *aUri,
 	NS_ADDREF(*aStyleSheet = styleSheet);
 	
 	return NS_OK;
-#else
-	return NS_ERROR_FAILURE;
 #endif
 }
 
 nsresult
 MozillaPrivate::RemoveOverrideStyleSheet (nsIDocShell *aDocShell,
+#if HAVE_NSISTYLESHEETSERVICE_H
+					  nsIURI *aUri)
+#else
 					  nsIStyleSheet* aStyleSheet)
+#endif
 {
+#if HAVE_NSISTYLESHEETSERVICE_H
+	nsresult rv;
+
+	nsCOMPtr<nsIStyleSheetService> service
+			(do_GetService ("@mozilla.org/content/style-sheet-service;1", &rv));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	PRBool isRegistered = PR_FALSE;
+	rv = service->SheetRegistered (aUri, nsIStyleSheetService::AGENT_SHEET,
+				       &isRegistered);
+	if (NS_SUCCEEDED (rv) && isRegistered)
+	{
+		rv = service->UnregisterSheet (aUri, nsIStyleSheetService::AGENT_SHEET);
+	}
+	if (NS_FAILED (rv))
+	{
+		g_warning ("Unregistering the user stylesheet failed (rv=%x)!\n", rv);
+	}
+
+	return rv;
+#else
 	nsCOMPtr<nsIPresShell> ps;
 	aDocShell->GetPresShell (getter_AddRefs(ps));
 	NS_ENSURE_TRUE (ps, NS_ERROR_FAILURE);
@@ -212,6 +258,7 @@ MozillaPrivate::RemoveOverrideStyleSheet (nsIDocShell *aDocShell,
 	aStyleSheet->SetEnabled(PR_FALSE);
 
 	return NS_OK;
+#endif
 }
 
 
