@@ -23,9 +23,7 @@
 #endif
 
 #include <nscore.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
 #include <nsCOMPtr.h>
 #include <nsIIOService.h>
 #include <nsIURI.h>
@@ -51,11 +49,7 @@
 #include "gul-general.h"
 #include "src/galeon-shell.h"
 
-#ifdef HAVE_CHECKLOADURI_CAPABILITIES
-# define FILE_PREFIX ""
-#else
-# define FILE_PREFIX "myportal:url:"
-#endif
+#define FILE_PREFIX ""
 
 static NS_DEFINE_CID(kSimpleURICID, NS_SIMPLEURI_CID);
 static NS_DEFINE_CID(kInputStreamChannelCID, NS_INPUTSTREAMCHANNEL_CID);
@@ -148,16 +142,7 @@ GMyportalProtocolHandler::NewChannel(nsIURI *aURI,
 			if (NS_SUCCEEDED (rv)) return NS_OK;
 		}
 
-#ifndef HAVE_CHECKLOADURI_CAPABILITIES
-		if (path.StartsWith ("url:"))
-		{
-			rv = RedirectURL (path.get() + 4, _retval);
-		}
-		else
-#endif
-		{
-			rv = CreateMyportalPage (path.get(), aURI, _retval);
-		}
+		rv = CreateMyportalPage (path.get(), aURI, _retval);
 	}
 	return rv;
 }
