@@ -34,9 +34,8 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
 #include <nsIWebBrowserPersist.h>
 #include <nsCWebBrowserPersist.h>
 #include <nsIFileURL.h>
@@ -52,18 +51,8 @@
 static const PRInt64 kInterval            = 500000;     /* in microsecs == 500ms == 0.5s */
 static const PRInt64 kUnknownSizeInterval = 150000;     /* in microsecs == 150ms == 0.15s */
 
-#ifdef HAVE_NSITRANSFER_H
-#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
 NS_IMPL_ISUPPORTS4(GProgressListener, nsIWebProgressListener, nsIWebProgressListener2,
 		   nsITransfer, nsISupportsWeakReference)
-#else
-NS_IMPL_ISUPPORTS3(GProgressListener, nsIWebProgressListener,
-		   nsITransfer, nsISupportsWeakReference)
-#endif
-#else
-NS_IMPL_ISUPPORTS4(GProgressListener, nsIWebProgressListener, 
-		   nsIDownload, nsITransfer, nsISupportsWeakReference)
-#endif
 
 //---------------------------------------------------------------------------
 
@@ -88,12 +77,7 @@ download_pause_cb (Download *download, GProgressListener *self)
 //---------------------------------------------------------------------------
 
 GProgressListener::GProgressListener () :
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 					  mCancelable(nsnull),
-#else
-					  mPersist(nsnull),
-					  mObserver(nsnull),
-#endif
 					  mGaleonPersist(nsnull),
 					  mDownload(nsnull),
 					  mContentLength(-1),
@@ -115,13 +99,7 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	/* fill in download details */
 	mUri = aURI;
 	mFile = aFile;
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 	mCancelable = aPersist;
-#else
-	mPersist = aPersist;
-	/* pick up progress messages */
-	mPersist->SetProgressListener (this);
-#endif
 
 	mGaleonPersist = (GaleonEmbedPersist *) g_object_ref (galeonPersist);
 
@@ -194,29 +172,15 @@ NS_METHOD GProgressListener::PrivateInit (void)
 
 NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
 				      nsIURI *aTarget,
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 				      const nsAString &aDisplayName,
-#else
-                                      const PRUnichar *aDisplayName,
-#endif
 				      nsIMIMEInfo *aMIMEInfo,
                                       PRInt64 aStartTime,
-#if HAVE_NSITRANSFER_TEMPFILE
 				      nsILocalFile *aTempFile,
-#endif
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 				      nsICancelable *aCancelable)
-#else
-                                      nsIWebBrowserPersist *aPersist)
-#endif
 {
         mUri = aSource;
         mStartTime = aStartTime;
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 	mCancelable = aCancelable;
-#else
-        mPersist = aPersist;
-#endif
 
 	mTarget = aTarget;
 	nsCOMPtr<nsIFileURL> tFileUrl = do_QueryInterface(mTarget);
@@ -229,7 +193,11 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
 
 	if (aMIMEInfo)
 	{
+#ifdef HAVE_NSHANDLERINFOACTION
+		nsHandlerInfoAction action;
+#else
 		nsMIMEInfoHandleAction action;
+#endif
 		if (NS_SUCCEEDED(aMIMEInfo->GetPreferredAction(&action)))
 		{
 			if (action == nsIMIMEInfo::useHelperApp)
@@ -246,113 +214,6 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
         return PrivateInit();
 }
 
-#ifndef HAVE_NSITRANSFER_H
-/* attribute long long startTime; */
-NS_IMETHODIMP GProgressListener::GetStartTime(PRInt64 *aStartTime)
-{
-        *aStartTime = mStartTime;
-        return NS_OK;
-}
-
-
-/* attribute nsIURI source; */
-NS_IMETHODIMP GProgressListener::GetSource(nsIURI * *aSource)
-{
-        NS_IF_ADDREF(*aSource = mUri);
-        return NS_OK;
-}
-
-/* attribute nsIURI target; */
-NS_IMETHODIMP GProgressListener::GetTarget(nsIURI * *aTarget)
-{
-	NS_IF_ADDREF(*aTarget = mTarget);
-	return NS_OK;
-}
-
-/* attribute nsILocalFile targetFile; */
-NS_IMETHODIMP GProgressListener::GetTargetFile(nsILocalFile * *aTarget)
-{
-        nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(mFile);
-        NS_IF_ADDREF(*aTarget = localFile);
-        return NS_OK;
-}
-
-NS_IMETHODIMP GProgressListener::GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo)
-{
-	NS_IF_ADDREF(*aMIMEInfo = mMIMEInfo);
-	return NS_OK;
-}
-
-/* readonly attribute PRInt32 percentComplete; */
-NS_IMETHODIMP GProgressListener::GetPercentComplete(PRInt32 *aPercentComplete)
-{
-	*aPercentComplete = 0;
-        return NS_OK;
-}
-
-#ifdef HAVE_NSIDOWNLOAD_GETSIZE
-/* readonly attribute PRUint64 amountTransferred; */
-NS_IMETHODIMP GProgressListener::GetAmountTransferred(PRUint64 *aAmountTransferred)
-{
-   *aAmountTransferred = 0;
-   return NS_OK;
-}
-
-
-/* readonly attribute PRUint64 size; */
-NS_IMETHODIMP GProgressListener::GetSize(PRUint64 *aSize)
-{
-   *aSize = 0;
-   return NS_OK;
-}
-#endif    
-
-/* attribute wstring displayName; */
-NS_IMETHODIMP GProgressListener::GetDisplayName(PRUnichar * *aDisplayName)
-{
-        *aDisplayName = nsnull;
-        return NS_OK;
-}
-NS_IMETHODIMP GProgressListener::SetDisplayName(const PRUnichar * aDisplayName)
-{
-        return NS_OK;
-}
-
-NS_IMETHODIMP GProgressListener::GetPersist(nsIWebBrowserPersist * *aPersist)
-{
-        NS_IF_ADDREF(*aPersist = mPersist);
-        return NS_OK;
-}
-
-#endif
-
-#ifndef HAVE_NSITRANSFER_NSICANCELABLE
-/* attribute nsIObserver observer; */
-NS_IMETHODIMP GProgressListener::GetObserver(nsIObserver * *aObserver)
-{
-        NS_IF_ADDREF(*aObserver = mObserver);
-        return NS_OK;
-}
-NS_IMETHODIMP GProgressListener::SetObserver(nsIObserver * aObserver)
-{
-        mObserver = aObserver;
-        return NS_OK;
-}
-#endif
-
-/* attribute nsIWebProgressListener listener; */
-#ifndef HAVE_NSIWEBPROGRESSLISTENER2_H
-NS_IMETHODIMP GProgressListener::GetListener(nsIWebProgressListener * *aListener)
-{
-        *aListener = nsnull;
-        return NS_OK;
-}
-NS_IMETHODIMP GProgressListener::SetListener(nsIWebProgressListener * aListener)
-{
-        return NS_OK;
-}
-#endif
-
 static void LaunchHandler (GnomeVFSMimeApplication *aHandler, nsIFile *aFile,
 			   guint32 user_time)
 {
@@ -418,16 +279,8 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
                 // so kungfu is needed to survive just long enough to do the
                 // final clean up.
                 nsCOMPtr<nsITransfer> kungFuDeathGrip(this);
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 		mCancelable = nsnull;
-#else
-                if (mPersist)
-                {
-                        mPersist->SetProgressListener(nsnull);
-                        mPersist = nsnull;
-                }
-#endif
-		
+
 		if (NS_SUCCEEDED(aStatus))
 		{
 			if (mAddToRecent)
@@ -542,11 +395,7 @@ NS_IMETHODIMP GProgressListener::
  *                          in long long aCurTotalProgress, 
  *                          in long long aMaxTotalProgress); 
  */
-#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
 NS_IMETHODIMP GProgressListener::
-#else
-NS_METHOD GProgressListener::
-#endif
 			OnProgressChange64 (nsIWebProgress *aWebProgress,
 					    nsIRequest *aRequest,
 					    PRInt64 aCurSelfProgress,
@@ -730,14 +579,7 @@ NS_METHOD GProgressListener::LaunchHelperApp (void)
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	GulString helperDesc;
-#ifdef HAVE_NSIMIMEINFO_NSASTRING
-	rv = mMIMEInfo->GetApplicationDescription(helperDesc);
-#else
-	PRUnichar *tmp;
-	rv = mMIMEInfo->GetApplicationDescription (&tmp);
-	helperDesc = tmp;
-	nsMemory::Free (tmp);
-#endif
+	rv = mMIMEInfo->GetDescription(helperDesc);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	// XXX information passing kludge (from ContentHandler)
@@ -819,7 +661,6 @@ nsresult GProgressListener::Abort (void)
                 mRequest->Resume ();
         }
 
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 	if (mCancelable)
 	{
 		nsresult rv;
@@ -831,18 +672,6 @@ nsresult GProgressListener::Abort (void)
 		   return rv;
 		}
 	}
-#else
-        if (mObserver)
-        {
-                mObserver->Observe(NS_ISUPPORTS_CAST(nsITransfer*, this),
-				   "oncancel", nsnull);
-        }
-
-        if (mPersist)
-        {
-                return mPersist->CancelSave ();
-        }
-#endif
 
 	// Working assumption is that if mPersist is set CancelSave correctly
 	// emits proper events to OnStateChange.  At the time of writing that
@@ -899,10 +728,8 @@ InitiateMozillaDownload (nsIURI *sourceURI, nsILocalFile* inDestFile,
 	rv = listener->InitForPersist (webPersist, displayURI, inDestFile, embedPersist);
 	NS_ENSURE_SUCCESS (rv, rv);
 
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 	rv = webPersist->SetProgressListener (listener);
 	NS_ENSURE_SUCCESS (rv, rv);
-#endif
 
 	PRInt32 persist_flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
 	if (!domDocument && !decode)
