@@ -31,11 +31,7 @@
 
 #include <gtk/gtkwidget.h>
 
-#ifdef HAVE_NSITRANSFER_H
 #include <nsITransfer.h>
-#else
-#include <nsIDownload.h>
-#endif
 #include <nsIWebProgressListener.h>
 #include <nsCOMPtr.h>
 #include <nsWeakReference.h>
@@ -65,30 +61,13 @@ class nsIInputStream;
 #define G_PROGRESSDIALOG_CONTRACTID "@mozilla.org/progressdialog;1"
 
 class GProgressListener : 
-#ifdef HAVE_NSITRANSFER_H
 			  public nsITransfer,
-#else
-			  public nsIDownload,
-			  public nsIWebProgressListener,
-#endif
  			  public nsSupportsWeakReference
 {
  public:
  	NS_DECL_ISUPPORTS
 	NS_DECL_NSIWEBPROGRESSLISTENER
-#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
 	NS_DECL_NSIWEBPROGRESSLISTENER2
-#else
-	NS_METHOD OnProgressChange64 (nsIWebProgress *aWebProgress,
-				      nsIRequest *aRequest,
-				      PRInt64 aCurSelfProgress,
-				      PRInt64 aMaxSelfProgress,
-				      PRInt64 aCurTotalProgress,
-				      PRInt64 aMaxTotalProgress);
-#endif
-#ifndef HAVE_NSITRANSFER_H
-	NS_DECL_NSIDOWNLOAD
-#endif
 	NS_DECL_NSITRANSFER
 
 	GProgressListener ();
@@ -108,12 +87,7 @@ class GProgressListener :
 	NS_METHOD LaunchHelperApp (void);
 	NS_METHOD SetRequest (nsIRequest *aRequest);
 
-#ifdef HAVE_NSITRANSFER_NSICANCELABLE
 	nsCOMPtr<nsICancelable> mCancelable;
-#else
-	nsCOMPtr<nsIWebBrowserPersist> mPersist;
-	nsCOMPtr<nsIObserver> mObserver;
-#endif
 	nsCOMPtr<nsIRequest> mRequest;
 	
 	GaleonEmbedPersist *mGaleonPersist;
