@@ -20,7 +20,6 @@
 #define __sidebarproxy_h
 
 #include <nsISidebar.h>
-#include <nsIClassInfo.h>
 #include <nsError.h>
 
 #define G_SIDEBAR_CLASSNAME \
@@ -29,11 +28,10 @@
 #define G_SIDEBAR_CID \
 { 0x7e38cd39, 0xb90d, 0x4756, { 0x97, 0x9c, 0x18, 0xea, 0x53, 0x1e, 0x3b, 0xea } }
 
-class GSidebarProxy : public nsISidebar, nsIClassInfo
+class GSidebarProxy : public nsISidebar
 {
   public:
 	NS_DECL_ISUPPORTS
-	NS_DECL_NSICLASSINFO
 	NS_DECL_NSISIDEBAR
 
 	GSidebarProxy();
