@@ -21,9 +21,9 @@
 #endif
 
 #include <nscore.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
+#include <nsXPCOM.h>
 
 #include "ProgressListener.h"
 #include "mozilla-embed-persist.h"
