@@ -71,6 +71,10 @@
 #ifdef HAVE_NSIPASSWORD_H
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
+#elif defined(HAVE_NSILOGINMANAGER_H)
+#include <nsILoginInfo.h>
+#include <nsILoginManager.h>
+#include <nsIIDNService.h>
 #endif
 #include <nsICookie.h>
 #include <nsCPasswordManager.h>
@@ -215,6 +219,17 @@ mozilla_init_profile (void)
         g_free (profile_path);
 }
 
+static void
+mozilla_init_login_manager(void)
+{
+#ifdef HAVE_NSILOGINMANAGER_H
+        nsCOMPtr<nsILoginManager> loginManager =
+                        do_GetService (NS_LOGINMANAGER_CONTRACTID);
+	if (!loginManager)
+		g_warning ("Failed to instantiate LoginManager");
+#endif
+}
+
 static void
 mozilla_init_plugin_path ()
 {
@@ -354,6 +369,7 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	  *lastSlash = '\0';
 
 	gtk_moz_embed_set_path(xpcomLocation);
+	gtk_moz_embed_set_comp_path(SHARE_DIR);
 #endif // XPCOM_GLUE
 #ifdef GALEON_MOZILLA_HOME
 	mozilla_init_home ();
@@ -362,16 +378,17 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	mozilla_init_plugin_path();
 	mozilla_init_profile ();
 
-	/* Fire up the beast */
-	gtk_moz_embed_push_startup ();
-
-
 #if HAVE_NSITOOLKITCHROMEREGISTRY_H
 	nsCOMPtr<nsIDirectoryServiceProvider> dp = new EphyDirectoryProvider ();
 	if (dp) {
 		gtk_moz_embed_set_directory_service_provider (dp);
 	}
 #endif
+
+	/* Fire up the beast */
+	gtk_moz_embed_push_startup ();
+
+	mozilla_init_login_manager();
 }
 
 
@@ -783,6 +800,126 @@ impl_list_passwords (GaleonEmbedShell *shell,
 		passwords = g_list_prepend (passwords, p);
         }       
 	return g_list_reverse (passwords);
+
+#elif defined(HAVE_NSILOGINMANAGER_H)
+	nsresult rv;
+	GList *passwords = NULL;
+	PRUint32 count,i;
+
+	nsCOMPtr<nsILoginManager> loginManager =
+			do_GetService (NS_LOGINMANAGER_CONTRACTID);
+	NS_ENSURE_TRUE (loginManager, NULL);
+
+	nsCOMPtr<nsIIDNService> idnService
+		(do_GetService ("@mozilla.org/network/idn-service;1"));
+	NS_ENSURE_TRUE (idnService, NULL);
+
+	if (type == PASSWORD_REJECT) {
+		PRUnichar **hostnames = nsnull;
+
+		rv = loginManager->GetAllDisabledHosts (&count, &hostnames);
+		NS_ENSURE_SUCCESS(rv, NULL);
+
+		for (i = 0; i < count; i++) {
+			nsCString host;
+			nsString transfer (hostnames[i]);
+			idnService->ConvertACEtoUTF8 (NS_ConvertUTF16toUTF8(transfer), host);
+
+	                PasswordInfo *p = g_new0 (PasswordInfo, 1);
+	                p->host = g_strdup(host.get());
+
+			passwords = g_list_prepend (passwords, p);
+		}
+
+		NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY (count, hostnames);
+	} else {
+		nsILoginInfo **logins = nsnull;
+		nsresult rv;
+
+		rv = loginManager->GetAllLogins (&count, &logins);
+		NS_ENSURE_SUCCESS(rv, NULL);
+
+		for (i=0; i < count; i++) {
+			nsString transfer;
+			nsString unicodeName;
+			rv = logins[i]->GetHostname (transfer);
+			if (NS_FAILED (rv)) continue;
+
+			nsCString host;
+			if (transfer.IsVoid())
+                  	host.SetIsVoid(PR_TRUE);
+                	else
+                  	idnService->ConvertACEtoUTF8 (NS_ConvertUTF16toUTF8(transfer), host);
+
+			rv = logins[i]->GetHttpRealm (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString httpRealm;
+			if (unicodeName.IsVoid())
+                  	httpRealm.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, httpRealm);
+
+			rv = logins[i]->GetUsername (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString userName;
+			if (unicodeName.IsVoid())
+                  	userName.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, userName);
+
+			rv = logins[i]->GetUsernameField (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString usernameField;
+			if (unicodeName.IsVoid())
+                  	usernameField.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, usernameField);
+
+			rv = logins[i]->GetPassword (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString userPassword;
+			if (unicodeName.IsVoid())
+                  	userPassword.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, userPassword);
+
+			rv = logins[i]->GetPasswordField (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString passwordField;
+			if (unicodeName.IsVoid())
+                  	passwordField.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, passwordField);
+
+			rv = logins[i]->GetFormSubmitURL (unicodeName);
+			if (NS_FAILED (rv)) continue;
+			nsCString formSubmitURL;
+			if (unicodeName.IsVoid())
+                  	formSubmitURL.SetIsVoid(PR_TRUE);
+                	else
+                  	NS_UTF16ToCString (unicodeName,
+                                    	NS_CSTRING_ENCODING_UTF8, formSubmitURL);
+
+			PasswordInfo *p = g_new0(PasswordInfo, 1);
+			p->host = host.IsVoid() ? NULL: g_strdup(host.get());
+			p->username = userName.IsVoid() ? NULL: g_strdup(userName.get());
+			p->password = userPassword.IsVoid() ? NULL: g_strdup(userPassword.get());
+			p->httpRealm = httpRealm.IsVoid() ? NULL : g_strdup(httpRealm.get());
+			p->usernameField = usernameField.IsVoid() ? NULL : g_strdup(usernameField.get());
+			p->passwordField = passwordField.IsVoid() ? NULL : g_strdup(passwordField.get());
+			p->formSubmitURL = formSubmitURL.IsVoid() ? NULL : g_strdup(formSubmitURL.get());
+
+			passwords = g_list_prepend (passwords, p);
+		}
+
+		NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY (count, logins);
+	}
+	return passwords;
 #else
 	return NULL;
 #endif
@@ -793,6 +930,7 @@ impl_remove_passwords (GaleonEmbedShell *shell,
 		       GList *passwords, 
 		       PasswordType type)
 {
+#ifdef HAVE_NSIPASSWORD_H
 	nsresult result = NS_ERROR_FAILURE;
         nsCOMPtr<nsIPasswordManager> passwordManager =
                         do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
@@ -813,6 +951,93 @@ impl_remove_passwords (GaleonEmbedShell *shell,
 
                 if (NS_FAILED(result)) return;
         };
+
+#elif defined (HAVE_NSILOGINMANAGER_H)
+      	nsCOMPtr<nsILoginManager> loginManager =
+		do_GetService (NS_LOGINMANAGER_CONTRACTID);
+	NS_ENSURE_TRUE (loginManager, );
+
+	if (type == PASSWORD_REJECT) {
+	        for (GList *l = passwords; l != NULL; l = l->next)
+        	{
+                	PasswordInfo *info = (PasswordInfo *)l->data;
+
+			nsString host;
+			NS_CStringToUTF16(nsCString(info->host),
+					  NS_CSTRING_ENCODING_UTF8, host);
+			loginManager->SetLoginSavingEnabled(host, TRUE);
+		}
+	} else {
+        	for (GList *l = passwords; l != NULL; l = l->next)
+        	{
+                	PasswordInfo *info = (PasswordInfo *)l->data;
+
+			nsString userName;
+
+	        	if (info->username)
+	          	NS_CStringToUTF16 (nsCString(info->username),
+        	                    	NS_CSTRING_ENCODING_UTF8, userName);
+        		else
+          		userName.SetIsVoid (PR_TRUE);
+
+			nsString host;
+			nsString userNameField;
+			nsString password;
+			nsString passwordField;
+			nsString httpRealm;
+			nsString formSubmitURL;
+
+        		if (info->host)
+          		NS_CStringToUTF16 (nsCString(info->host),
+                            		NS_CSTRING_ENCODING_UTF8, host);
+        		else
+          		host.SetIsVoid (PR_TRUE);
+
+        		if (info->usernameField)
+          		NS_CStringToUTF16 (nsCString(info->usernameField),
+                            		NS_CSTRING_ENCODING_UTF8, userNameField);
+        		else
+          		userNameField.SetIsVoid (PR_TRUE);
+
+        		if (info->httpRealm)
+          		NS_CStringToUTF16 (nsCString(info->httpRealm),
+                            		NS_CSTRING_ENCODING_UTF8, httpRealm);
+        		else
+          		httpRealm.SetIsVoid (PR_TRUE);
+
+        		if (info->password)
+          		NS_CStringToUTF16 (nsCString(info->password),
+                            		NS_CSTRING_ENCODING_UTF8, password);
+        		else
+          		password.SetIsVoid (PR_TRUE);
+
+        		if (info->passwordField)
+          		NS_CStringToUTF16 (nsCString(info->passwordField),
+                            		NS_CSTRING_ENCODING_UTF8, passwordField);
+        		else
+          		passwordField.SetIsVoid (PR_TRUE);
+
+        		if (info->formSubmitURL)
+          		NS_CStringToUTF16 (nsCString(info->formSubmitURL),
+                            		NS_CSTRING_ENCODING_UTF8, formSubmitURL);
+        		else
+          		formSubmitURL.SetIsVoid (PR_TRUE);
+
+			nsCOMPtr<nsILoginInfo> login
+				(do_CreateInstance(NS_LOGININFO_CONTRACTID));
+
+			login->SetUsername(userName);
+			login->SetUsernameField(userNameField);
+			login->SetHostname(host);
+			login->SetHttpRealm(httpRealm);
+			login->SetFormSubmitURL(formSubmitURL);
+			login->SetPassword(password);
+			login->SetPasswordField(passwordField);
+
+			loginManager->RemoveLogin(login);
+		}
+	}
+#endif
 }
 
 static GList *
