@@ -1813,7 +1813,17 @@ impl_remove_user_stylesheet(GaleonEmbed *embed,
 
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 
+#ifdef HAVE_NSISTYLESHEETSERVICE_H
+	gchar *path = galeon_embed_utils_get_user_sheet_path(sheet->name);
+
+	gchar *tmp = g_strconcat("file://", path, NULL);
+	g_free(path);
+
+	wrapper->RemoveOverrideStyleSheet(tmp);
+	g_free(tmp);
+#else
 	wrapper->RemoveOverrideStyleSheet(mess->style);
+#endif
 }
 
 EmbedStyleSheet *
