@@ -501,11 +501,11 @@ save_item (const char *key, GaleonFaviconCacheEntry *entry, xmlTextWriterPtr wri
 			info->on_disk = TRUE;
 		}
 
-		xmlTextWriterStartElement (writer, "entry");
+		xmlTextWriterStartElement (writer, (const xmlChar*)"entry");
 
-		xmlTextWriterWriteAttribute (writer, "url", key);
-		xmlTextWriterWriteAttribute (writer, "favicon",
-					     entry->priv->info->pixbuf_filename);
+		xmlTextWriterWriteAttribute (writer, (const xmlChar*)"url", (const xmlChar*)key);
+		xmlTextWriterWriteAttribute (writer, (const xmlChar*)"favicon",
+					     (const xmlChar*)entry->priv->info->pixbuf_filename);
 
 		xmlTextWriterEndElement (writer);
 	}
@@ -519,7 +519,7 @@ galeon_favicon_cache_save_to_file (GaleonFaviconCache *cache,
 
 	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
 	if (ret < 0) return ret;
-	ret = xmlTextWriterStartElement (writer, "GaleonFaviconCache");
+	ret = xmlTextWriterStartElement (writer, (const xmlChar*)"GaleonFaviconCache");
 	if (ret < 0) return ret;
 	
 	g_hash_table_foreach (cache->priv->url_to_entry, (GHFunc) save_item, writer);
@@ -586,7 +586,7 @@ galeon_favicon_cache_sync_to_disc (GaleonFaviconCache *cache)
 		int ret;
 
 		xmlTextWriterSetIndent (writer, 1);
-		xmlTextWriterSetIndentString (writer, "  ");
+		xmlTextWriterSetIndentString (writer, (const xmlChar*)"  ");
 
 		ret = galeon_favicon_cache_save_to_file (cache, writer);
 		if (ret < 0)
@@ -620,8 +620,8 @@ galeon_favicon_cache_sync_from_disc (GaleonFaviconCache *cache)
 
 	for (child = doc->children->children; child != NULL; child = child->next)
 	{
-		char *url = xmlGetProp (child, "url");
-		char *favicon = xmlGetProp (child, "favicon");
+		char *url = (char*)xmlGetProp (child, (const xmlChar*)"url");
+		char *favicon = (char*)xmlGetProp (child, (const xmlChar*)"favicon");
 
 		if (url && favicon)
 		{
@@ -756,7 +756,7 @@ persist_stream_cb (GaleonEmbedPersist *persist, const char *buffer,
 	GdkPixbufLoader *loader = data;
 	GError *error = NULL;
 	
-	if (!gdk_pixbuf_loader_write (loader, buffer, len, &error))
+	if (!gdk_pixbuf_loader_write (loader,(guchar*) buffer, len, &error))
 	{
 		LOG("error=%s", error->message);
 		g_error_free (error);
