@@ -592,6 +592,7 @@ galeon_prefs_web_content_construct (GaleonPrefsDialog *self, GladeXML *gxml)
  * Privacy page
  */
 void on_prefs_privacy_certificates_manage_button_clicked (GtkWidget *button, GaleonPrefsDialog *self);
+void on_prefs_privacy_devices_manage_button_clicked (GtkWidget *button, GaleonPrefsDialog *self);
 void on_prefs_privacy_cookies_manage_button_clicked      (GtkWidget *button, GaleonPrefsDialog *self);
 void on_prefs_privacy_passwords_manage_button_clicked    (GtkWidget *button, GaleonPrefsDialog *self);
 
@@ -602,6 +603,12 @@ on_prefs_privacy_certificates_manage_button_clicked (GtkWidget *button, GaleonPr
 	g_signal_emit (G_OBJECT(self), signals[REQUEST], 0, GALEON_PREFS_MANAGE_CERTIFICATES);
 }
 void
+on_prefs_privacy_devices_manage_button_clicked (GtkWidget *button, GaleonPrefsDialog *self)
+{
+	g_return_if_fail (GALEON_IS_PREFS_DIALOG(self));
+	g_signal_emit (G_OBJECT(self), signals[REQUEST], 0, GALEON_PREFS_MANAGE_DEVICES);
+}
+void
 on_prefs_privacy_cookies_manage_button_clicked (GtkWidget *button, GaleonPrefsDialog *self)
 {
 	g_return_if_fail (GALEON_IS_PREFS_DIALOG(self));
