@@ -25,11 +25,12 @@
 #include "gul-general.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
+#include "gul-x11.h"
 
 #include <string.h>
 #include <time.h>
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkdialog.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -66,6 +67,10 @@ static void session_create_bookmarks (Session *session, const char *filename);
 
 static GObjectClass *parent_class = NULL;
 
+#define SESSION_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_SESSION, SessionPrivate))
+
+
 struct SessionPrivate
 {
 	GaleonDialog *crash_dialog;
@@ -154,6 +159,8 @@ session_class_init (SessionClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	g_type_class_add_private (klass, sizeof (SessionPrivate));
 }
 
 static char *
@@ -206,7 +213,7 @@ do_session_resume (Session *session, int type)
 	switch (type)
 	{
 	case RESTORE_SESSION:
-		session_load (session, crashed_session);
+		session_load (session, crashed_session, 0u);
 		break;
 	case RESTORE_AS_BOOKMARKS:
 		session_create_bookmarks (session, crashed_session);
@@ -214,6 +221,7 @@ do_session_resume (Session *session, int type)
 	case DISCARD_SESSION:
 		break;
 	}
+	g_free (crashed_session);
 }
 
 static gboolean
@@ -288,7 +296,7 @@ session_autoresume (Session *session)
 	    && !loaded
 	    && eel_gconf_get_boolean (CONF_GENERAL_ALWAYS_SAVE_SESSION))
 	{
-		session_load (session, saved_session);
+		session_load (session, saved_session, 0u);
 	}
 	
 	g_free (saved_session);
@@ -297,26 +305,6 @@ session_autoresume (Session *session)
 	return (session->priv->windows != NULL) ? SESSION_RESUMED : SESSION_OK;
 }
 
-static void
-create_session_directory (void)
-{
-	char *dir;
-
-	dir = g_build_filename (g_get_home_dir(),
-			        GALEON_DIR"/sessions", 
-			        NULL);
-	
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS))
-	{
-		if (mkdir (dir, 488) != 0)
-                {
-                        g_error ("couldn't make %s' directory", dir);
-                }
-	}
-
-	g_free (dir);
-}
-
 static gboolean 
 save_yourself_cb  (GnomeClient *client,
                    gint phase,
@@ -326,21 +314,32 @@ save_yourself_cb  (GnomeClient *client,
                    gboolean fast,
 		   Session *session)
 {
-	char *argv[] = { "galeon", "--load-session", NULL };
+	char *argv[4] = { "galeon", NULL, NULL, NULL };
 	char *discard_argv[] = { "rm", "-r", NULL };
-	
-	argv[2] = get_session_filename (SESSION_GNOME);
-	gnome_client_set_restart_command 
-		(client, 3, argv);
-     
-	discard_argv[2] = argv[2];
-     	gnome_client_set_discard_command (client, 3, 
-					  discard_argv);
-			
-	session_save (session, argv[2]);
+	char *session_file = NULL;
+	int count = 1;
 
-	g_free (argv[2]);
-	
+	if (galeon_shell_get_server_mode (galeon_shell))
+	{
+		argv[count++] = "--server";
+	}
+
+	if (session_get_windows (session))
+	{
+		session_file = get_session_filename (SESSION_GNOME);
+		discard_argv[2] = session_file;
+		gnome_client_set_discard_command (client, 3, discard_argv);
+
+		argv[count++] = "--load-session";
+		argv[count++] = session_file;
+
+		session_save (session, session_file);
+	}
+
+	gnome_client_set_restart_command (client, count, argv);
+
+	g_free (session_file);
+     
 	return TRUE;
 }
 
@@ -348,7 +347,14 @@ static void
 session_die_cb (GnomeClient* client, 
 		Session *session)
 {
+	g_object_ref (galeon_shell);
+
 	session_close (session);
+
+	/* Turn off server mode, so that we exit nicely */
+	galeon_shell_set_server_mode (galeon_shell, FALSE);
+
+	g_object_unref (galeon_shell);
 }
 
 static void
@@ -372,13 +378,11 @@ gnome_session_init (Session *session)
 static void
 session_init (Session *session)
 {
-        session->priv = g_new0 (SessionPrivate, 1);
+        session->priv = SESSION_GET_PRIVATE (session);
 	session->priv->windows = NULL;
 	session->priv->dont_remove_crashed = FALSE;
 	session->priv->session_saved = FALSE;
 	
-	create_session_directory ();
-
 	gnome_session_init (session);
 }
 
@@ -450,7 +454,6 @@ session_finalize (GObject *object)
 
 	g_list_free (t->priv->windows);
 	
-        g_free (t->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -467,7 +470,7 @@ session_new (void)
 {
 	Session *t;
 
-	t = SESSION (g_object_new (SESSION_TYPE, NULL));
+	t = SESSION (g_object_new (TYPE_SESSION, NULL));
 
 	g_return_val_if_fail (t->priv != NULL, NULL);
 
@@ -492,15 +495,15 @@ save_tab (GaleonWindow *window,
                         
 	/* make a new XML node */
         embed_node = xmlNewDocNode (doc, NULL,
-                                    "embed", NULL);
+                                    (const xmlChar*)"embed", NULL);
 
         /* store title in the node */
 	title = galeon_tab_get_title (tab);
-	xmlSetProp (embed_node, "title", title);
+	xmlSetProp (embed_node, (const xmlChar*)"title", (const xmlChar*)title);
 	
         /* otherwise, use the actual location. */
 	location = galeon_tab_get_location (tab);
-        xmlSetProp (embed_node, "url", location);
+        xmlSetProp (embed_node,(const xmlChar*) "url", (const xmlChar*)location);
 	
 	/* insert node into the tree */
 	xmlAddChild (window_node, embed_node);	
@@ -527,12 +530,19 @@ session_save (Session *session,
 	
 	g_return_if_fail (IS_SESSION (session));
 
+	/* Dont save empty sessions, just delete the session file */
+	if (!session->priv->windows)
+	{
+		session_delete (session, filename);
+		return;
+	}
+
 	save_to = get_session_filename (filename);
 		
-        doc = xmlNewDoc ("1.0");
+        doc = xmlNewDoc ((const xmlChar*)"1.0");
 
         /* create and set the root node for the session */
-        root_node = xmlNewDocNode (doc, NULL, "session", NULL);
+        root_node = xmlNewDocNode (doc, NULL, (const xmlChar*)"session", NULL);
         xmlDocSetRootElement (doc, root_node);
 
         /* iterate through all the windows */
@@ -540,6 +550,7 @@ session_save (Session *session,
         {
 		GList *tabs, *l;
         	int x = 0, y = 0, width = 0, height = 0;
+		const char *role;
 		GaleonWindow *window = GALEON_WINDOW(w->data);
 		GtkWidget *wmain;
 		
@@ -547,7 +558,7 @@ session_save (Session *session,
 		g_return_if_fail (tabs != NULL);
 		
                 /* make a new XML node */
-                window_node = xmlNewDocNode (doc, NULL, "window", NULL);
+                window_node = xmlNewDocNode (doc, NULL, (const xmlChar*)"window", NULL);
 
                 /* get window geometry */
 		wmain = GTK_WIDGET (window);
@@ -555,15 +566,22 @@ session_save (Session *session,
                 gtk_window_get_position (GTK_WINDOW(wmain), &x, &y);
 
                 /* set window properties */
-                snprintf(buffer, 32, "%d", x);
-                xmlSetProp (window_node, "x", buffer);
-                snprintf(buffer, 32, "%d", y);
+                g_snprintf(buffer, 32, "%d", x);
+                xmlSetProp (window_node, (const xmlChar*)"x", (const xmlChar*)buffer);
+                g_snprintf(buffer, 32, "%d", y);
 
-		xmlSetProp (window_node, "y", buffer);
-                snprintf(buffer, 32, "%d", width);
-                xmlSetProp (window_node, "width", buffer);
-                snprintf(buffer, 32, "%d", height);
-                xmlSetProp (window_node, "height", buffer);
+		xmlSetProp (window_node, (const xmlChar*)"y", (const xmlChar*)buffer);
+                g_snprintf(buffer, 32, "%d", width);
+                xmlSetProp (window_node, (const xmlChar*)"width", (const xmlChar*)buffer);
+                g_snprintf(buffer, 32, "%d", height);
+                xmlSetProp (window_node, (const xmlChar*)"height", (const xmlChar*)buffer);
+
+		role = gtk_window_get_role (GTK_WINDOW (window));
+		if (role != NULL)
+		{
+			xmlSetProp (window_node, (const xmlChar*)"role", 
+				    (const xmlChar *)role);
+		}
 
 		for (l = tabs; l != NULL; l = l->next)
 	        {
@@ -576,7 +594,7 @@ session_save (Session *session,
         }
 
         /* save it all out to disk */
-        xmlSaveFile (save_to, doc);
+	gul_general_safe_xml_save (save_to, doc);
         xmlFreeDoc (doc);
 
 	g_free (save_to);
@@ -585,6 +603,7 @@ session_save (Session *session,
 typedef void (*WindowCb) (xmlNodePtr child, 
 			  const int x, const int y, 
 			  const int width, const int height, 
+			  const char *role,
 			  gpointer data);
 typedef void (*TabCb) (const char *url, const char *title, 
 		       gpointer data);
@@ -595,14 +614,14 @@ parse_embed (xmlNodePtr child, TabCb tab_cb, gpointer data)
 {	
 	while (child != NULL)
 	{
-		if (strcmp (child->name, "embed") == 0)
+		if (xmlStrcmp (child->name, (const xmlChar*)"embed") == 0)
 		{
-			char *url;
-			char *title;
+			xmlChar* url;
+			xmlChar* title;
 			
-			url = xmlGetProp (child, "url");
-			title = xmlGetProp (child, "title");
-			tab_cb (url, title, data);
+			url = xmlGetProp (child, (const xmlChar*)"url");
+			title = xmlGetProp (child, (const xmlChar*)"title");
+			tab_cb ((char*)url, (char*)title, data);
 			xmlFree (url);
 			xmlFree (title);
 		}
@@ -623,23 +642,31 @@ parse_xml (Session *session,
 	save_to = get_session_filename (filename);
 
 	doc = xmlParseFile (save_to);
+	if (doc == NULL)
+	{
+		return;
+	}
 
 	child = xmlDocGetRootElement (doc); 
+	g_return_if_fail (child!= NULL);
 
 	/* skip the session node */
 	child = child->children;
 	
 	while (child != NULL)
 	{
-		if (strcmp (child->name, "window") == 0)
+		if (xmlStrcmp (child->name, (const xmlChar*)"window") == 0)
 		{
 			gint x = 0, y = 0, width = 0, height = 0;
-			
+			xmlChar *role;
+
 			x = xmlGetIntProp (child, "x");
                         y = xmlGetIntProp (child, "y");
                         width = xmlGetIntProp (child, "width");
                         height = xmlGetIntProp (child, "height");
-			window_cb (child, x, y, width, height, data);
+			role = xmlGetProp (child, (const xmlChar*)"role");
+			window_cb (child, x, y, width, height, (const char*)role, data);
+			xmlFree (role);
 		}
 
 		child = child->next;
@@ -660,9 +687,10 @@ create_new_tab (const char *url, const char *title, GaleonWindow *window)
 	tab = galeon_tab_new ();	
 	embed = galeon_tab_get_embed (tab);
 	
-	gtk_widget_show (GTK_WIDGET(embed));
+	gtk_widget_show (GTK_WIDGET(tab));
 	
-	galeon_window_add_tab (window, tab, FALSE);
+	galeon_window_add_tab (window, tab,
+			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
 	
 	galeon_embed_load_url (embed, url);
 }
@@ -670,19 +698,29 @@ create_new_tab (const char *url, const char *title, GaleonWindow *window)
 static void
 create_new_window (xmlNodePtr child,
 		   const int x, const int y, 
-		   const int width, const int height)
+		   const int width, const int height,
+		   const char * role,
+		   gpointer data)
 {
 	GtkWidget *wmain;
 	GaleonWindow *window;
 
 	window = galeon_window_new ();
 	wmain = GTK_WIDGET (window);
-	gtk_widget_show (GTK_WIDGET(window));
+
+	gul_x11_window_update_user_time (wmain, GPOINTER_TO_INT (data));
 	
 	gtk_window_move (GTK_WINDOW(wmain), x, y);
 	gtk_window_set_default_size (GTK_WINDOW (wmain),
 				     width, height);
 
+	if (role)
+	{
+		gtk_window_set_role (GTK_WINDOW (window), role);
+	}
+
+	gtk_widget_show (GTK_WIDGET(window));
+
 	parse_embed (child->children, (TabCb)create_new_tab, window);
 }
 
@@ -702,12 +740,14 @@ bookmark_from_tab (const char *url, const char *title, GbFolder *folder)
 	set = galeon_shell_get_bookmark_set (galeon_shell);
 	new_site = gb_site_new (set, title, url);
 	gb_folder_add_child (folder, GB_BOOKMARK(new_site), -1);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (new_site));
 }
 
 static void
 bookmarks_from_window (xmlNodePtr child,
 		       const int x, const int y, 
 		       const int width, const int height,
+		       const char *role,
 		       GbFolder *folder)
 {
 	g_return_if_fail (GB_IS_FOLDER (folder));
@@ -723,12 +763,13 @@ bookmarks_from_window (xmlNodePtr child,
  **/
 
 void 
-session_load (Session *session, const char *filename)
+session_load (Session *session, const char *filename, guint32 user_time)
 {
-	parse_xml (session, filename, (WindowCb)create_new_window, NULL);
+	parse_xml (session, filename, (WindowCb)create_new_window,
+		   GINT_TO_POINTER(user_time));
 }
 
-const GList *
+GList *
 session_get_windows (Session *session)
 {
 	g_return_val_if_fail (IS_SESSION (session), NULL);
@@ -741,14 +782,13 @@ session_get_windows (Session *session)
  * in a subfolder of the root folder called "Sites opened before
  * crash on <date> at <time>"
  */
-#define DATE_LENGTH 30
 static
 void session_create_bookmarks (Session *session, const char *filename)
 {
 	GbFolder *root_folder;
 	GbFolder *new_folder;
 	GbBookmarkSet *set;
-	char date[DATE_LENGTH];
+	char date[64];
 	const char *date_hack="%c"; /* This is necessary to avoid a warning */
 	struct tm *tm;
 	time_t timep;
@@ -764,8 +804,8 @@ void session_create_bookmarks (Session *session, const char *filename)
 
 	time (&timep);
 	tm = localtime (&timep);
-	if (strftime (date, DATE_LENGTH, date_hack, tm) != 0) {
-		gchar *date_utf8 = g_locale_to_utf8 (date, DATE_LENGTH, 
+	if (strftime (date, sizeof(date), date_hack, tm) != 0) {
+		gchar *date_utf8 = g_locale_to_utf8 (date, -1,
 						     NULL, NULL, NULL);
 		
 		folder_name = g_strdup_printf (_("Sites opened before crash on %s"), date_utf8);
@@ -775,8 +815,9 @@ void session_create_bookmarks (Session *session, const char *filename)
 	}
 	new_folder =  gb_folder_new (set, folder_name);
 	g_free (folder_name);
-	gb_folder_add_child (root_folder, GB_BOOKMARK(new_folder), -1);
-	
+	gb_folder_add_child (root_folder, GB_BOOKMARK (new_folder), -1);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (new_folder));
+
 	/* Add bookmarks corresponding to the previous session to this folder
 	 */
 	parse_xml (session, filename, (WindowCb)bookmarks_from_window, 
@@ -820,9 +861,12 @@ session_remove_window (Session *session,
 
 	session->priv->windows = g_list_remove (session->priv->windows, window);
 
+	session_save (session, SESSION_CRASHED);
+
 	/* autodestroy of the session, necessay to avoid
 	 * conflicts with the nautilus view */
-	if (session->priv->windows == NULL)
+	if (session->priv->windows == NULL && 
+	    galeon_shell_get_server_mode (galeon_shell) == FALSE)
 	{
 		g_object_unref (session);
 	}
