@@ -22,7 +22,6 @@
 
 #include "galeon-autocompletion-source.h"
 #include "gul-filesystem-autocompletion.h"
-#include "gul-gobject-misc.h"
 #include "galeon-debug.h"
 #include <string.h>
 
@@ -31,6 +30,10 @@
 /**
  * Private data
  */
+#define GUL_FILESYSTEM_AUTOCOMPLETION_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GUL_TYPE_FILESYSTEM_AUTOCOMPLETION, GulFilesystemAutocompletionPrivate))
+
+
 struct _GulFilesystemAutocompletionPrivate {
 	gchar *current_dir;
 	gchar *base_dir;
@@ -65,10 +68,11 @@ static gpointer g_object_class;
 /**
  * FilesystemAutocompletion object
  */
-MAKE_GET_TYPE_IFACE (gul_filesystem_autocompletion, "GulFilesystemAutocompletion", GulFilesystemAutocompletion,
-		     gul_filesystem_autocompletion_class_init, gul_filesystem_autocompletion_init, G_TYPE_OBJECT,
-		     gul_filesystem_autocompletion_autocompletion_source_init, GALEON_TYPE_AUTOCOMPLETION_SOURCE);
-
+G_DEFINE_TYPE_WITH_CODE (GulFilesystemAutocompletion,gul_filesystem_autocompletion, 
+			 G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GALEON_TYPE_AUTOCOMPLETION_SOURCE,
+			       gul_filesystem_autocompletion_autocompletion_source_init));
+			 
 static void
 gul_filesystem_autocompletion_autocompletion_source_init (GaleonAutocompletionSourceIface *iface)
 {
@@ -82,12 +86,14 @@ gul_filesystem_autocompletion_class_init (GulFilesystemAutocompletionClass *klas
 	G_OBJECT_CLASS (klass)->finalize = gul_filesystem_autocompletion_finalize_impl;
 
 	g_object_class = g_type_class_peek_parent (klass);
+
+	g_type_class_add_private (klass, sizeof (GulFilesystemAutocompletionPrivate));
 }
 
 static void 
 gul_filesystem_autocompletion_init (GulFilesystemAutocompletion *e)
 {
-	GulFilesystemAutocompletionPrivate *p = g_new0 (GulFilesystemAutocompletionPrivate, 1);
+	GulFilesystemAutocompletionPrivate *p = GUL_FILESYSTEM_AUTOCOMPLETION_GET_PRIVATE (e);
 	e->priv = p;
 
 	p->score = G_MAXINT / 2;
@@ -111,7 +117,6 @@ gul_filesystem_autocompletion_finalize_impl (GObject *o)
 		gnome_vfs_uri_unref (p->base_dir_uri);
 	}
 
-	g_free (p);
 
 	G_OBJECT_CLASS (g_object_class)->finalize (o);
 }
