@@ -2,6 +2,7 @@
 #define CONF_GENERAL_HOMEPAGE "/apps/galeon/Browsing/General/startpage"
 #define CONF_GENERAL_ALWAYS_SAVE_SESSION "/apps/galeon/Browsing/General/always_save_session"
 #define CONF_GENERAL_NEWPAGE_TYPE "/apps/galeon/Browsing/General/newpage_type"
+#define CONF_GENERAL_CONFIRM_CLOSE "/apps/galeon/Browsing/General/confirm_close"
 #define CONF_GENERAL_OFFLINE "/apps/galeon/Browsing/General/offline"
 
 /* Bookmarks */
@@ -40,8 +41,6 @@
 #define CONF_TOOLBAR_STYLE "/apps/galeon/UI/Toolbar/toolbar_override_style"
 
 /* External apps */
-#define CONF_PROGRAMS_GNOME_MAILER "/apps/galeon/Handlers/Programs/gnome_mailer"
-#define CONF_PROGRAMS_MAILER "/apps/galeon/Handlers/Programs/mailer"
 #define CONF_DOWNLOADING_ASK_DIR "/apps/galeon/Handlers/Downloading/ask_for_download_dir"
 #define CONF_DOWNLOADING_SHOW_DETAILS "/apps/galeon/Handlers/Downloading/show_details"
 #define CONF_DOWNLOADING_EXTERNAL_DOWNLOADER "/apps/galeon/Handlers/Downloading/external_downloader"
